/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.components.jdbc;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.concurrent.atomic.AtomicReference;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.sql.DataSource;
import javax.xml.namespace.NamespaceContext;

import org.apache.commons.dbutils.DbUtils;
import org.apache.commons.dbutils.QueryRunner;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Node;
import org.dom4j.dom.DOMDocument;
import org.dom4j.tree.DefaultDocument;
import org.eclipse.ebpm.components.jdbc.config.JDBCQueryConfig;
import org.eclipse.ebpm.components.jdbc.config.QueryParameterConfig;
import org.eclipse.ebpm.components.jdbc.config.XmlResultSetHandler;
import org.eclipse.ebpm.core.BaseSpagicService;
import org.eclipse.ebpm.core.ExchangeUtils;
import org.eclipse.ebpm.datasource.IDataSourceManager;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.messaging.api.Message;
import org.eclipse.ebpm.util.properties.PropertyConfigurator;
import org.eclipse.ebpm.xmlutil.DOM4JUtils;
import org.eclipse.ebpm.xmlutil.EXMLUtils;
import org.eclipse.ebpm.xmlutil.MapNamespaceContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class JDBCComponent extends BaseSpagicService {
	
	Logger log = LoggerFactory.getLogger(JDBCComponent.class);
	
	
	protected JDBCQueryConfig queryConfig = null;
	protected String dsIdentifier = null;
	
	protected final AtomicReference<IDataSourceManager> datasourceManager = new AtomicReference<IDataSourceManager>(null); 
	
	public IDataSourceManager getDataSourceManager() {
		return this.datasourceManager.get();
	}
	public void setDataSourceManager(IDataSourceManager dsm) {
		this.datasourceManager.set(dsm);
	}
	
	public void unsetDataSourceManager(IDataSourceManager dsManager) {
		this.datasourceManager.compareAndSet(dsManager, null);
	}
	
	@Override
	public void init() {
		
		try{
			this.queryConfig = new JDBCQueryConfig();
			
			queryConfig.setQuery(propertyConfigurator.getString("query"));
			dsIdentifier = propertyConfigurator.getString("datasource");
			
			queryConfig.setColumnNameAsAttribute(propertyConfigurator.getBoolean("result.columnAsAttribute", false));
			queryConfig.setRowsXmlEnvelope(propertyConfigurator.getString("result.rowsXmlEnvelope", "rows"));
			queryConfig.setRowXmlEnvelope(propertyConfigurator.getString("result.rowsXmlEnvelope", "row"));
			queryConfig.setEnrichMessage(propertyConfigurator.getBoolean("result.enrichInputMessage", false));
			queryConfig.setValueAsAttribute(propertyConfigurator.getBoolean("result.valueAsAttribute", false));
			queryConfig.setXmlEnvelope(propertyConfigurator.getString("result.xmlEnvelope", ""));
			queryConfig.setFaultManagement(propertyConfigurator.getString("faultManagement", JDBCQueryConfig.FAULT_SYSTEM));

			Map<String, Properties> queryParamsMap = (Map<String, Properties>)propertyConfigurator.getXMapProperty("queryParametersMap");
			Properties pOfParam = null;
			PropertyConfigurator pConfigOfParam = null;
			
			QueryParameterConfig qpConfig = null;
			ArrayList<QueryParameterConfig> paramList = new ArrayList<QueryParameterConfig>();
			
			for ( String key : queryParamsMap.keySet()){
				pOfParam = queryParamsMap.get(key);
				pConfigOfParam = new PropertyConfigurator(pOfParam);
				
				qpConfig = new QueryParameterConfig(
								key, 
								pConfigOfParam.getString("xPathExpression"), 
								pConfigOfParam.getString("parameterType"));
				qpConfig.setOutputParam(pConfigOfParam.getBoolean("outputParam",false));
				paramList.add(qpConfig);
			}
			
			queryConfig.setQueryParams(paramList);


		}catch (Exception e) {
			log.error(e.getMessage(), e);
		}


	}

	public JDBCQueryConfig getQueryConfig() {
		return queryConfig;
	}

	public void setQueryConfig(JDBCQueryConfig config) {
		this.queryConfig = config;
	}

	@Override
	public boolean run(Exchange exchange, Message in, Message out)
			throws Exception {
		
		log.info("JDBC Component ["+getSpagicId()+"] process Exchange Method");	
		
		log.info("JDBC Component ["+getSpagicId()+"] Retrieving Datasource from Datasource Manager");	
		
		DataSource ds = getDataSourceManager().getDataSource(dsIdentifier);
	    List<String> orderedParameterNameList = getStringParameterNames(queryConfig.getQuery(), queryConfig.getMatchParamRegexp());

	    String jdbcQuery = queryConfig.getQuery().replaceAll(queryConfig.getMatchParamRegexp(), "?");

	    List<QueryParameterConfig> queryParametersConfig = queryConfig.getQueryParams();

	    Map<String, QueryParameterConfig> parameterConfigMap = buildParameterConfigMap(queryParametersConfig);

	    Object[] params = queryConfig.getParamsFromInXml(in, orderedParameterNameList, parameterConfigMap);
	 // execute query
		// get connection
		Connection conn = ds.getConnection();
		Element results = null;
		try {
			QueryRunner runner = new QueryRunner();
			log.debug("Executing query: " + jdbcQuery);
			try {
				if (jdbcQuery.toLowerCase().indexOf("select") != -1) {
					results = (Element) runner.query(conn, jdbcQuery, params, 
							new XmlResultSetHandler(queryConfig.getRowsXmlEnvelope(), 
									queryConfig.getRowXmlEnvelope(), 
									queryConfig.getColumnNameAsAttribute(),
									queryConfig.getValueAsAttribute()));
				} else {
					int intResult = runner.update(conn, jdbcQuery, params);
					Element updateResult = DocumentHelper.createElement("affected-rows");
					updateResult.setText("" + intResult);
					results = updateResult;
				}
			} catch (SQLException sqle) {
				if (JDBCQueryConfig.FAULT_FLOW.equals(queryConfig.getFaultManagement())) {
					
					Element faultElem = DocumentHelper.createElement("Fault");
					Element sqlStateElem = DocumentHelper.createElement("sql-state");
					sqlStateElem.setText(sqle.getSQLState());
					Element errorCodeElem = DocumentHelper.createElement("error-code");
					errorCodeElem.setText("" + sqle.getErrorCode());
					faultElem.add(sqlStateElem);
					faultElem.add(errorCodeElem);
					if (ExchangeUtils.isInAndOut(exchange)){
						exchange.getFault(true).setBody(faultElem.asXML());
					}
					
					out.setBody(faultElem.asXML());
					return true;
				}else{
					throw sqle;
				}
			}
			log.debug("Query results: " + results);
		} finally {
			
			DbUtils.close(conn);
		
		}

		// compose out response
		// 1) the results is a single row. The elements putted in some other dom
		// element
		// 2) more rows. Must be setted as is

		out.setBody(in.getBody());
		
		String strContent = in.getBodyText();
		Document docContent = DocumentHelper.parseText(strContent);
		NamespaceContext nsContextMap = DOM4JUtils.getNamespaceContext(docContent);
		org.w3c.dom.Document docContentDOM = DOM4JUtils.dom4j_to_dom(docContent);
		EXMLUtils xmlUtils = new EXMLUtils(nsContextMap);
		
		
		// search the envelop where to put the result
		org.w3c.dom.Element contentEnvelop = null;

		if (isValidStr(queryConfig.getXmlEnvelope())) {
			contentEnvelop = xmlUtils.selectNode(docContentDOM,"/" + queryConfig.getXmlEnvelope());
			if (contentEnvelop == null) {
				log.warn("Envelope to enrich not found [" + "/" + queryConfig.getXmlEnvelope() + "]: "
						+ "the result will be the original message");
			}
		} else {
			log.info("Xml Envelope is null we will keep the original message ");
		}

		if (contentEnvelop != null && queryConfig.isEnrichMessage() && isValidStr(queryConfig.getXmlEnvelope())) {
			//
			// Case 1: enrich message = true, xmlenvelop != null
			// Result is Original message + result attached as child of
			// (queryConfig.getXmlEnvelope())
			//
			log.debug("Enriching Original Message, JDBC Result are children of queryConfig.getXmlEnvelope()");
			String resultXML = results.asXML();
			org.w3c.dom.Document resultsDOM = xmlUtils.parseText(resultXML);
			
			if (results.elements().size() == 1) {
				log.debug("Result has exactly 1 row");
			
				StringBuffer rowXpath = new StringBuffer("/").append(queryConfig.getRowsXmlEnvelope()).append("/").append(
						queryConfig.getRowXmlEnvelope());
				
				org.w3c.dom.Element row = xmlUtils.selectNode(resultsDOM,rowXpath.toString());

				if (row != null) {
					log.debug("Adding result to envelope: ");
					org.w3c.dom.Node importedNode = docContentDOM.importNode(row, true);

					((org.w3c.dom.Element) contentEnvelop).appendChild(((org.w3c.dom.Element) row));
				} else {
					log.warn("Envelope with single row result not found [" + rowXpath + "]: "
							+ "all the results will be putted content envelop");
					
					org.w3c.dom.Node importedNode = docContentDOM.importNode(resultsDOM.getDocumentElement(), true);
					((org.w3c.dom.Element) contentEnvelop).appendChild(importedNode);
				}
			} else {
				org.w3c.dom.Node importedNode = docContentDOM.importNode(resultsDOM.getDocumentElement(), true);
				((org.w3c.dom.Element) contentEnvelop).appendChild(importedNode);

			}
			out.setBody(DOM4JUtils.dom_to_dom4j(docContentDOM).asXML());

		} else if (contentEnvelop == null && queryConfig.isEnrichMessage()) {
			//
			// Case 2: enrich message = true, xmlenvelop == null
			// Result will be the original message
			//
			log.debug("Enriching =  true, xmlEnvelope is null -> Keep The Original Message");
			out.setBody(docContent.asXML());
		} else {
			//
			// That is the case in which we do not keep original message but we create
			// a new
			//
			log.debug("Not enriching... Keep only the output result");
			docContent = new DOMDocument();
			docContent.add(results);
			out.setBody(docContent.asXML());
		}

		return true;
	}	






	public List<String> getStringParameterNames(String propertyValue, String matchRegexp) {
        Pattern pattern = Pattern.compile(matchRegexp);
        Matcher matcher = 
        	pattern.matcher(propertyValue);
        String placeHolder;
        List<String> toReturn = new ArrayList<String>();
		for (int idx = 1; matcher.find(); idx++) {
			placeHolder = matcher.group();
			placeHolder = placeHolder.substring(1);
			toReturn.add(placeHolder);
		}
		return toReturn;
	}

	private Map<String, QueryParameterConfig> buildParameterConfigMap(
			List<QueryParameterConfig> queryParametersConfig) {
		Map<String, QueryParameterConfig> mapParamConfig = new HashMap<String, QueryParameterConfig>();
		for (QueryParameterConfig q : queryParametersConfig) {
			mapParamConfig.put(q.getPlaceHolder(), q);
		}
		return mapParamConfig;
	}

	protected boolean isValidStr(String s) {
		return s != null && s.trim().length() > 0;
	}

	
}
