/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.connectors.filesystem.reader;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Date;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.ConcurrentHashMap;

import org.eclipse.ebpm.connectors.filesystem.adapters.IFileAdapter;
import org.eclipse.ebpm.core.AbstractSpagicConnector;
import org.eclipse.ebpm.core.routing.IMessageRouter;
import org.eclipse.ebpm.dirwatcher.DirWatcher;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.messaging.api.Message;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class FileSystemPollerConnector extends AbstractSpagicConnector{

		protected Logger logger = LoggerFactory.getLogger(FileSystemPollerConnector.class);
		
		private Timer pollingScheduler = null;
		private TimerTask pollingTask = null;
		private File directory;
	    private String extensionFilter;
	    private boolean deleteFile = true;
	   
	 
	    private ConcurrentHashMap<String, IFileAdapter> marshalers = new ConcurrentHashMap<String, IFileAdapter>();
	    private String marshalerId;
	    private ConcurrentHashMap<String, InputStream> openedStream = new ConcurrentHashMap<String, InputStream>();
	    private ConcurrentHashMap<String, File> openedFiles = new ConcurrentHashMap<String, File>();
	    
	    public void init() {
	    	String directoryString = propertyConfigurator.getString("directory");
	    	this.directory = new File(directoryString);
	    	this.extensionFilter = propertyConfigurator.getString("filter", null);
	    	this.marshalerId = propertyConfigurator.getString("marshaller", "DEFAULT");
		}
	   
		public void start() throws Exception {
			super.start();
			String[] filters = null;
			if (extensionFilter != null)
				filters = new String[] {extensionFilter};
			else
				filters = new String[0];
					
			this.pollingTask  = new DirWatcher(directory, filters) {
				protected void onChange(File file, String action) {
					try{
						if (action.equalsIgnoreCase("add")){
//							String uri = file.toURI().relativize(file.toURI()).toString();
							IFileAdapter adapter = marshalers.get(marshalerId);
							if (!file.isDirectory())
								processFile(file,adapter);
						}
					}catch(Exception e){
						logger.error(e.getMessage(), e);
					}		
				}
			};

			pollingScheduler = new Timer();
			pollingScheduler.schedule(pollingTask, new Date(), 5000);
			
		}
		
		public void stop() throws Exception {
			super.stop();
			pollingScheduler.cancel();
			
		}
		
		public void process(Exchange exchange) {
			String exchangeId = exchange.getId();
			
			InputStream is = openedStream.remove(exchangeId);
			if (is == null){
				throw new Error("Cannot Get The Opened File For Stream for Exchange" + exchangeId);
			}
			try{
				is.close();
			}catch (Exception e) {
				throw new IllegalStateException(e.getMessage(), e);
			}
			
			
			File f = openedFiles.remove(exchangeId);
			
			if (f == null)
				throw new IllegalStateException("Cannot Get The Opened file Stream for Exchange" + exchangeId);
			
			if (deleteFile){
				if (!f.delete()){
					  throw new IllegalStateException("Could not delete file " + f);
				}
			}
			
			
			
		}
		
		public void processFile(File f, IFileAdapter adapter ) {
			try{
				
				// Check if the target service is available
		        IMessageRouter router = getMessageRouter();
		        if (!router.isServiceRegistered(this.target)){
		        	logger.warn("Target Service ["+this.target+"] is not Registered, cannot fire message");
		        	return;
		        }				
				
				InputStream stream = new BufferedInputStream(new FileInputStream(f));
				Exchange exchange = createInOnlyExchange();
				Message inMessage = exchange.getIn(true);
				
				
				adapter.readFile(exchange, inMessage, stream, f.getCanonicalPath());
			
				
				this.openedStream.put(exchange.getId(), stream);
				this.openedFiles.put(exchange.getId(), f);
	        
				send(exchange);
				stream.close();
			}catch (Exception e) {
				logger.error("Error in Process File " + e.getMessage(), e);
			}
			}
			
			public void setMarshaler (IFileAdapter adapter) {
				this.marshalers.put(adapter.getAdapterId(), adapter);
			}	
			public void unsetMarshaler(IFileAdapter marshaler) {
				this.marshalers.remove(marshaler.getAdapterId());
			}
		
}
