/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.connectors.jms.endpoints;

import java.util.Dictionary;
import java.util.concurrent.atomic.AtomicReference;

import javax.jms.Connection;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.MessageConsumer;
import javax.jms.MessageListener;
import javax.jms.Queue;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.Topic;
import javax.jms.TopicSession;

import org.eclipse.ebpm.connectors.jms.marshaler.DefaultConsumerMarshaler;
import org.eclipse.ebpm.connectors.jms.marshaler.JmsConsumerMarshaler;
import org.eclipse.ebpm.constants.SpagicConstants;
import org.eclipse.ebpm.core.AbstractSpagicConnector;
import org.eclipse.ebpm.jms.connectionfactory.ConnectionFactory;
import org.eclipse.ebpm.jms.connectionfactory.IConnectionFactoryManager;
import org.eclipse.ebpm.util.properties.PropertyConfigurator;
import org.osgi.service.component.ComponentContext;
import org.osgi.service.event.Event;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class JmsConsumerEndpoint extends AbstractSpagicConnector {

	protected final AtomicReference<IConnectionFactoryManager> connectionFactoryManager = new AtomicReference<IConnectionFactoryManager>(null); 

	private JmsConsumerMarshaler marshaler = new DefaultConsumerMarshaler();
    private boolean pubSubDomain;
    protected String connectionFactoryName;
    private ConnectionFactory connectionFactory;
	private Connection queueConnection;
	private Session session;
	private MessageConsumer receiver;
	private MessageListener listener;

	protected Logger logger = LoggerFactory.getLogger(JmsConsumerEndpoint.class);

	// Standard jms properties
	private Destination destination;
	private String destinationName;
	private String messageSelector;

	public JmsConsumerEndpoint() {
		super();
	}

	/* (non-Javadoc)
	 * @see org.spagic3.core.AbstractSpagicService#init()
	 */
	public void init() {

		this.connectionFactoryName = propertyConfigurator.getString("connectionFactoryName");
		try {
			// If the connection factory is not available, don't worry, simply wait
			connectionFactory = getConnectionFactoryManager().getConnectionFactory(connectionFactoryName);
		} catch (Exception e) {
			logger.warn(e.getMessage());
		}		

		this.pubSubDomain = propertyConfigurator.getBoolean("pubSubDomain");
		
		// Retrieve the queue name
		this.destinationName = propertyConfigurator.getString("destinationName");
	}

	public IConnectionFactoryManager getConnectionFactoryManager() {
		return this.connectionFactoryManager.get();
	}
	
	public void setConnectionFactoryManager(IConnectionFactoryManager cfm) {
		this.connectionFactoryManager.set(cfm);
	}
	
	public void unsetConnectionFactoryManager(IConnectionFactoryManager cfm) {
		this.connectionFactoryManager.compareAndSet(cfm, null);
	}

    /**
     * @return the connectionFactory
     */
    public ConnectionFactory getConnectionFactory() {
        return connectionFactory;
    }

    /**
    * Specifies the <code>ConnectionFactory</code> used by the endpoint.
    *
     * @param connectionFactory the connectionFactory to set
     */
    public void setConnectionFactory(ConnectionFactory connectionFactory) {
        this.connectionFactory = connectionFactory;
    }

    /**
     * @return the pubSubDomain
     */
    public boolean isPubSubDomain() {
        return pubSubDomain;
    }

    /**
    * Specifies if the destination is a topic. <code>true</code> means the 
    * destination is a topic. <code>false</code> means the destination is a 
    * queue.
    *
     * @param pubSubDomain the destination is a topic?
     */
    public void setPubSubDomain(boolean pubSubDomain) {
        this.pubSubDomain = pubSubDomain;
    }

    /**
     * @return the marshaler
     */
    public JmsConsumerMarshaler getMarshaler() {
        return marshaler;
    }

    /**
    * Specifies the class implementing the message marshaler. The message 
    * marshaller is responsible for marshalling and unmarshalling JMS messages. 
    * The default is <code>DefaultConsumerMarshaler</code>.
    *
     * @param marshaler the marshaler implementation
     */
    public void setMarshaler(JmsConsumerMarshaler marshaler) {
        this.marshaler = marshaler;
    }
    
	/**
	 * @return the destination
	 */
	public Destination getDestination() {
		return destination;
	}

	/**
	 * Specifies the JMS <code>Destination</code> used to receive messages.
	 * 
	 * @param destination
	 *            the JMS destination
	 * @see org.springframework.jms.listener.AbstractMessageListenerContainer#setDestination(Destination)
	 */
	public void setDestination(Destination destination) {
		this.destination = destination;
	}

	/**
	 * @return the destinationName
	 */
	public String getDestinationName() {
		return destinationName;
	}

	/**
	 * Specifies a string identifying the JMS destination used to recieve
	 * messages. The destination is resolved using the
	 * <code>DesitinationResolver</code>.
	 * 
	 * @param destinationName
	 *            the destination name
	 * @see org.springframework.jms.listener.AbstractMessageListenerContainer#setDestinationName(String)
	 */
	public void setDestinationName(String destinationName) {
		this.destinationName = destinationName;
	}

	/**
	 * @return the messageSelector
	 */
	public String getMessageSelector() {
		return messageSelector;
	}

	/**
	 * Specifies the message selector string to use. The message selector string
	 * should conform to the descrition in the JMS spec.
	 * 
	 * @param messageSelector
	 *            the message selector string
	 * @see org.springframework.jms.listener.AbstractMessageListenerContainer#setMessageSelector(String)
	 */
	public void setMessageSelector(String messageSelector) {
		this.messageSelector = messageSelector;
	}

	public void start() throws Exception {
		validate();

		if (getConnectionFactory()==null)
			// Connection factory still not available. Wait until an internal event notify
			// the availability of the factory
			return;		
	        
		try {
			queueConnection = connectionFactory.createConnection();
			session = queueConnection.createSession(false, Session.AUTO_ACKNOWLEDGE);
			destination = connectionFactory.getDestination(pubSubDomain, destinationName);

			if (pubSubDomain) {
				receiver = ((TopicSession)session).createSubscriber((Topic)destination);
			} else {
				receiver = ((QueueSession)session).createReceiver((Queue)destination);
			}
			
			queueConnection.start();

            // Start polling
			listener = new TextListener(this, getMarshaler());
            receiver.setMessageListener(listener);
            
			super.start();
		} catch (Exception e) {
			logger.error("Unable to start JMS connector", e);
			stop();
		}		
	}

	public void stop() throws Exception {
		if (receiver != null)
			try {
				receiver.close();
			} catch (JMSException e1) {
				logger.error("Problems during JMS connector shutdown", e1);
			} finally {
				receiver = null;
			}
		if (session != null)
			try {
				session.close();
			} catch (JMSException e1) {
				logger.error("Problems during JMS connector shutdown", e1);
			} finally {
				session = null;
			}
		if (queueConnection != null) {
			try {
				queueConnection.stop();
				queueConnection.close();
			} catch (JMSException e1) {
				logger.error("Problems during JMS connector shutdown", e1);
			} finally {
				queueConnection = null;
			}
		}

		super.stop();
	}

	public void validate() {
		if (destination == null && destinationName == null) {
			throw new RuntimeException("destination or destinationName is required");
		}
	}
	
	@Override
	public void handleInternalEvent(Event event) {
		logger.info("JMSConsumerEndpoint["+getSpagicId()+"] handle internal Event Start Event ["+event+"]");
		String evType =(String)event.getProperty(SpagicConstants._INTERNAL_EVENT_TYPE);
		if (evType.equalsIgnoreCase(SpagicConstants._INTERNAL_EVENT_CF_DEPLOYED)){
			String id =(String)event.getProperty(SpagicConstants._INTERNAL_EVENT_REFERRING_ID);
			logger.info("JMSConsumerEndpoint["+getSpagicId()+"]::ConnectionFactory ["+id+"] Registered on ConnectionFactoryManager");
			if (id.equals(connectionFactoryName)){
				logger.info("JMSConsumerEndpoint["+getSpagicId()+"]:: Reacting to ConnectionFactory["+id+"] registration");
				synchronized (this) {
					try{
						setConnectionFactory(getConnectionFactoryManager().getConnectionFactory(connectionFactoryName));
						
			            // Begin polling
						start();						
					}catch (Exception e) {
						logger.error(e.getMessage(), e);
					}
				}
			}else{
				logger.info("JMSConsumerEndpoint["+getSpagicId()+"]:: Discard ConnectionFactory["+id+"] registration, interested in ["+connectionFactoryName+"]");
				return;
			}
		}else if (evType.equalsIgnoreCase(SpagicConstants._INTERNAL_EVENT_CF_UNDEPLOYED)){
			String id = (String)event.getProperty(SpagicConstants._INTERNAL_EVENT_REFERRING_ID);
			logger.info("JMSConsumerEndpoint["+getSpagicId()+"]::ConnectionFactory ["+id+"] UnRegistered on ConnectionFactoryManager");
			if (id.equals(connectionFactoryName)){
				logger.info("JMSConsumerEndpoint["+getSpagicId()+"]::Reacting to ConnectionFactory["+id+"] UnRegistration");
				synchronized (this) {
					 
					try {
						// Stop polling
						stop();
					} catch (Exception e) {
						logger.error(e.getMessage(), e);
					}
					
				}
			}else{
				logger.info("JMSConsumerEndpoint["+getSpagicId()+"]::Discard ConnectionFactory["+id+"] UnRegistration, interested in ["+connectionFactoryName+"]");
				return;
			}
		}
			
	}
	
	protected void activate(ComponentContext componentContext){
		// This code is usually performed by the superclass, but we need
		// the spagicId before the super.activate is called
		Dictionary propertiesDictionary = componentContext.getProperties();
		this.propertyConfigurator = new PropertyConfigurator(propertiesDictionary);
		this.spagicId = (String)this.propertyConfigurator.getString("spagic.id");
		
		// Register for notification of connection factory availability
		IConnectionFactoryManager cfManager = getConnectionFactoryManager();
		cfManager.registerForNotification(getSpagicId());
		
		// This should be the last statement of the method because it calls the init()
		super.activate(componentContext);
	}
}
