/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.connectors.jms.endpoints;

import java.util.Dictionary;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.atomic.AtomicReference;

import javax.jms.Connection;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageProducer;
import javax.jms.Session;

import org.eclipse.ebpm.connectors.jms.marshaler.DefaultProviderMarshaler;
import org.eclipse.ebpm.connectors.jms.marshaler.JmsProviderMarshaler;
import org.eclipse.ebpm.constants.SpagicConstants;
import org.eclipse.ebpm.core.AbstractSpagicConnector;
import org.eclipse.ebpm.jms.connectionfactory.ConnectionFactory;
import org.eclipse.ebpm.jms.connectionfactory.IConnectionFactoryManager;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.util.properties.PropertyConfigurator;
import org.osgi.service.component.ComponentContext;
import org.osgi.service.event.Event;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class JmsProviderEndpoint extends AbstractSpagicConnector {

	protected  final AtomicReference<IConnectionFactoryManager> connectionFactoryManager = new AtomicReference<IConnectionFactoryManager>(null); 

    private JmsProviderMarshaler marshaler;

    private String connectionFactoryName;
    private String destinationName;
    private boolean pubSubDomain;
    private boolean messageIdEnabled = true;
    private boolean messageTimestampEnabled = true;
    private boolean needJavaIdentifiers;    
    
    private ConnectionFactory connectionFactory;
    private Connection connection = null;
    private Session session = null;
    private Destination destination;
    private MessageProducer producer = null;
    private ConcurrentLinkedQueue<Exchange> waitingExchanges = new ConcurrentLinkedQueue<Exchange>();
        
	protected Logger logger = LoggerFactory.getLogger(JmsProviderEndpoint.class);

	@Override
	public void init() {
		this.connectionFactoryName = propertyConfigurator.getString("connectionFactoryName");
		try {
			connectionFactory = getConnectionFactoryManager().getConnectionFactory(connectionFactoryName);
		} catch (Exception e) {
			logger.warn(e.getMessage());
		}		

		this.destinationName = propertyConfigurator.getString("destinationName");
		this.messageIdEnabled = propertyConfigurator.getBoolean("messageIdEnabled", true);
		this.messageTimestampEnabled = propertyConfigurator.getBoolean("messageTimestampEnabled", true);
		this.needJavaIdentifiers = propertyConfigurator.getBoolean("needJavaIdentifiers", true);
		this.pubSubDomain = propertyConfigurator.getBoolean("pubSubDomain", true);
	}

	public IConnectionFactoryManager getConnectionFactoryManager() {
		return this.connectionFactoryManager.get();
	}
	
	public void setConnectionFactoryManager(IConnectionFactoryManager cfm) {
		this.connectionFactoryManager.set(cfm);
	}
	
	public void unsetConnectionFactoryManager(IConnectionFactoryManager cfm) {
		this.connectionFactoryManager.compareAndSet(cfm, null);
	}

	/**
     * @return the destination
     */
    public Destination getDestination() {
        return destination;
    }

    /**
    * Specifies the JMS <code>Destination</code> used to send messages.
    *
     * @param destination the destination
     */
    public void setDestination(Destination destination) {
        this.destination = destination;
    }

    /**
     * @return the destinationName
     */
    public String getDestinationName() {
        return destinationName;
    }

    /**
    * Specifies a string identifying the JMS destination used to send
     * messages. The destination is resolved using the
     * <code>DesitinationResolver</code>.
     *
     * @param destinationName the destination name
     */
    public void setDestinationName(String destinationName) {
        this.destinationName = destinationName;
    }

    /**
     * @return the connectionFactory
     */
    public ConnectionFactory getConnectionFactory() {
    	if (connectionFactory == null){
    		synchronized (this) {
				if (connectionFactory==null){
					try {
						connectionFactory = getConnectionFactoryManager().getConnectionFactory(connectionFactoryName);
					} catch (Exception e) {
						logger.warn(e.getMessage());
					}
				}
			}
    	}
        return connectionFactory;
    }

    /**
    * Specifies the <code>ConnectionFactory</code> used by the endpoint.
    *
     * @param connectionFactory the connectionFactory to set
     */
    public void setConnectionFactory(ConnectionFactory connectionFactory) {
        this.connectionFactory = connectionFactory;
    }

    /**
     * @return the marshaler
     */
    public JmsProviderMarshaler getMarshaler() {
        return marshaler;
    }

    /**
    * Specifies the class implementing the message marshaler. The message
    * marshaller is responsible for marshalling and unmarshalling JMS messages.
    * The default is <code>DefaultProviderMarshaler</code>.
    *
     * @param marshaler the marshaler implementation
     */
    public void setMarshaler(JmsProviderMarshaler marshaler) {
        if (marshaler == null) {
            throw new NullPointerException("marshaler is null");
        }
        this.marshaler = marshaler;
        marshaler.setNeedJavaIdentifiers(needJavaIdentifiers);
    }

    /**
     * @return the messageIdEnabled
     */
    public boolean isMessageIdEnabled() {
        return messageIdEnabled;
    }

    /**
    * Specifies if your endpoint requires JMS message IDs. Setting the
    * <code>messageIdEnabled</code> property to <code>false</code> causes the
    * endpoint to call its message producer's
    * <code>setDisableMessageID() </code> with a value of <code>true</code>.
    * The JMS broker is then given a hint that it does not need to generate
    * message IDs or add them to the messages from the endpoint. The JMS
    * broker can choose to accept the hint or ignore it.
    *
     * @param messageIdEnabled the endpoint requires message IDs?
     */
    public void setMessageIdEnabled(boolean messageIdEnabled) {
        this.messageIdEnabled = messageIdEnabled;
    }

    /**
     * @return the messageTimestampEnabled
     */
    public boolean isMessageTimestampEnabled() {
        return messageTimestampEnabled;
    }

    /**
    * Specifies if your endpoints requires time stamps on its messages.
    * Setting the <code>messageTimeStampEnabled</code> property to
    * <code>false</code> causes the endpoint to call its message producer's
    * <code>setDisableMessageTimestamp() </code> method with a value of
    * <code>true</code>. The JMS broker is then given a hint that it does not
    * need to generate message IDs or add them to the messages from the
    * endpoint. The JMS broker can choose to accept the hint or ignore it.
    *
     * @param messageTimestampEnabled the endpoint requires time stamps?
     */
    public void setMessageTimestampEnabled(boolean messageTimestampEnabled) {
        this.messageTimestampEnabled = messageTimestampEnabled;
    }

    /**
     * @return the pubSubDomain
     */
    public boolean isPubSubDomain() {
        return pubSubDomain;
    }

    /**
    * Specifies if the destination is a topic. <code>true</code> means the
    * destination is a topic. <code>false</code> means the destination is a
    * queue.
    *
     * @param pubSubDomain the destination is a topic?
     */
    public void setPubSubDomain(boolean pubSubDomain) {
        this.pubSubDomain = pubSubDomain;
    }

    @Override
	public boolean run(Exchange exchange,
			org.eclipse.ebpm.messaging.api.Message in,
			org.eclipse.ebpm.messaging.api.Message out) throws Exception {

        if (getConnectionFactory() == null) {
        	logger.info("Connection Factory still not available. Putting exchange in temporary queue");
    		waitingExchanges.add(exchange);
    		return false;
    	}

    	logger.info("Sending message with id: " + exchange.getId());

        // Create message and send it
        final Message message = marshaler.createMessage(exchange, in, session);
        producer.send(message);
		
		return true;
	}

    /**
     * Validate this endpoint.
     *
     * @throws DeploymentException
     */
    public void validate() {
        if (marshaler == null) {
            setMarshaler(createDefaultMarshaler());
        }
        
		if (destination == null && destinationName == null) {
			throw new RuntimeException("destination or destinationName is required");
		}
    }    
    
    /**
     * Creates the default marshaler for the endpoint if no custom marshaler has been set
     *
     * @return the default marshaler implementation
     */
    protected JmsProviderMarshaler createDefaultMarshaler() {
        return new DefaultProviderMarshaler();
    }

	@Override
	public void start() throws Exception {
		validate();
		
		if (getConnectionFactory()==null)
			// Connection factory still not available. Wait until an internal event notify
			// the availability of the factory
			return;		
		
		connection = getConnectionFactory().createConnection();
		session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
        destination = getConnectionFactory().getDestination(pubSubDomain, destinationName);
        producer = session.createProducer(destination);
        
        producer.setDisableMessageTimestamp(!messageTimestampEnabled);
        producer.setDisableMessageID(!messageIdEnabled);
		
		super.start();		
	}

	@Override
	public void stop() throws Exception {
		if (producer != null)
			try {
				producer.close();
			} catch (JMSException e1) {
				logger.error("Problems during JMS connector shutdown", e1);
			} finally {
				producer = null;
			}
		if (session != null)
			try {
				session.close();
			} catch (JMSException e1) {
				logger.error("Problems during JMS connector shutdown", e1);
			} finally {
				session = null;
			}
		if (connection != null) {
			try {
				connection.stop();
				connection.close();
			} catch (JMSException e1) {
				logger.error("Problems during JMS connector shutdown", e1);
			} finally {
				connection = null;
			}
		}

		super.stop();
	}
	
	public void handleInternalEvent(Event event) {
		logger.info("JMSProviderEndpoint["+getSpagicId()+"] handle internal Event Start Event ["+event+"]");
		String evType =(String)event.getProperty(SpagicConstants._INTERNAL_EVENT_TYPE);
		if (evType.equalsIgnoreCase(SpagicConstants._INTERNAL_EVENT_CF_DEPLOYED)){
			String id =(String)event.getProperty(SpagicConstants._INTERNAL_EVENT_REFERRING_ID);
			logger.info("JMSProviderEndpoint["+getSpagicId()+"]::ConnectionFactory ["+id+"] Registered on ConnectionFactoryManager");
			try{
				if (id.equals(connectionFactoryName)){
					logger.info("JMSProviderEndpoint["+getSpagicId()+"]::Reacting to ConnectionFactory["+id+"] registration");
					synchronized (this) {
						connectionFactory = getConnectionFactoryManager().getConnectionFactory(connectionFactoryName);
						start();
						
						logger.info("Consuming " + waitingExchanges.size() + " stored exchanges");
						Exchange storedExchange = null;
						while ((storedExchange = waitingExchanges.poll()) != null) {
							logger.info("Consuming exchange with id:" + storedExchange.getId());
							run(storedExchange, storedExchange.getIn(), storedExchange.getOut());
							done(storedExchange);							
						}
					}
				}else{
					logger.info("JMSProviderEndpoint["+getSpagicId()+"]::Discard ConnectionFactory["+id+"] registration, interested in ["+connectionFactoryName+"]");
					return;
				}
			}catch (Exception e) {
				logger.warn(e.getMessage());
			}
		}else if (evType.equalsIgnoreCase(SpagicConstants._INTERNAL_EVENT_CF_UNDEPLOYED)){
			String id = (String)event.getProperty(SpagicConstants._INTERNAL_EVENT_REFERRING_ID);
			logger.info("JMSProviderEndpoint["+getSpagicId()+"]::ConnectionFactory ["+id+"] UnRegistered on ConnectionFactoryManager");
			if (id.equals(connectionFactoryName)){
				logger.info("JMSProviderEndpoint["+getSpagicId()+"]::Reacting to ConnectionFactory["+id+"] UnRegistration");
				synchronized (this) {
					try {
						stop();
					} catch (Exception e) {
						logger.error("JMSProviderEndpoint["+getSpagicId()+"]::Error stopping component");
					}
				}
			}else{
				logger.info("JMSProviderEndpoint["+getSpagicId()+"]::Discard ConnectionFactory["+id+"] UnRegistration, interested in ["+connectionFactoryName+"]");
				return;
			}
		}
		
		
	}

	
	protected void activate(ComponentContext componentContext){
		// This code is usually performed by the superclass, but we need
		// the spagicId before the super.activate is called
		Dictionary propertiesDictionary = componentContext.getProperties();
		this.propertyConfigurator = new PropertyConfigurator(propertiesDictionary);
		this.spagicId = (String)this.propertyConfigurator.getString("spagic.id");
		
		// Register for notification of connection factory availability
		IConnectionFactoryManager cfManager = getConnectionFactoryManager();
		cfManager.registerForNotification(getSpagicId());
		
		// This should be the last statement of the method because it calls the init()
		super.activate(componentContext);
	}
	
}
