/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/

package org.eclipse.ebpm.connectors.mail;

import java.lang.reflect.Constructor;
import java.util.Properties;

import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.MimeMessage;

import org.eclipse.ebpm.connectors.mail.marshaler.AbstractMailMarshaler;
import org.eclipse.ebpm.connectors.mail.marshaler.DefaultMailMarshaler;
import org.eclipse.ebpm.connectors.mail.utils.MailConnectionConfiguration;
import org.eclipse.ebpm.connectors.mail.utils.MailUtils;
import org.eclipse.ebpm.core.AbstractSpagicConnector;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.messaging.api.Message;
import org.eclipse.ebpm.messaging.api.Status;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * this is the sending mail component
 * 
 * 
 * @author lhein for original sources
 * @author Rossato Luca for further modifications
 */

public class MailSenderComponent extends AbstractSpagicConnector {
	protected Logger log = LoggerFactory.getLogger(MailSenderComponent.class);
	
	public static final String PROPERTY_MARSHALER = "marshaler";
	public static final String PROPERTY_CUSTOMTRUSTMANAGER = "customTrustManagers";
	public static final String PROPERTY_SENDER = "sender";
	public static final String PROPERTY_SUBJECT = "subject";
	public static final String PROPERTY_RECEIVER = "receiver";
	public static final String PROPERTY_DEBUGMODE = "debugMode";
	public static final String PROPERTY_STARTTLS = "startTLS";	

	private AbstractMailMarshaler marshaler = new DefaultMailMarshaler();
    private String customTrustManagers;   
    private MailConnectionConfiguration config;
    private String connection;
    private String sender;   
    private String subject;
    private String receiver;
    private boolean debugMode;     
    private boolean startTLS = false;
	
		
	public void init() {
				
		AbstractMailMarshaler abstractMailMarshaler; 
		if(propertyConfigurator.getString(PROPERTY_MARSHALER)!=null && !propertyConfigurator.getString(PROPERTY_MARSHALER).equals("")){
			try{
				 Class classe = Class.forName(propertyConfigurator.getString(PROPERTY_MARSHALER));
				 Constructor c = classe.getConstructor(new Class[0]);
				 abstractMailMarshaler = (AbstractMailMarshaler)c.newInstance(new Object[0]);
			}catch(Exception e){
				log.error("The selected mail Marshaler is invalid: " + propertyConfigurator.getString(PROPERTY_MARSHALER) +
						" the defaul one will be used ", e);
				abstractMailMarshaler= new DefaultMailMarshaler();
			}
		}
		else
			abstractMailMarshaler = new DefaultMailMarshaler();
		setMarshaler(abstractMailMarshaler);
		
		String host = propertyConfigurator.getString("host");
		String password = propertyConfigurator.getString("password");
		String protocol = propertyConfigurator.getString("protocol");
		String username = propertyConfigurator.getString("username");
		int port = propertyConfigurator.getInteger("port", 25);
		boolean startTLS = propertyConfigurator.getBoolean("startTLS",false);		
		String folderName = propertyConfigurator.getString("folderName","INBOX");	
	
		MailConnectionConfiguration config = new MailConnectionConfiguration(host,password,protocol,username,startTLS,folderName,port);
		setConfig(config);
		setCustomTrustManagers(propertyConfigurator.getString(PROPERTY_CUSTOMTRUSTMANAGER,null));
		setConnection(config);
		setSender(propertyConfigurator.getString(PROPERTY_SENDER, ""));
		setSubject(propertyConfigurator.getString(PROPERTY_SUBJECT, ""));
		setReceiver(propertyConfigurator.getString(PROPERTY_RECEIVER));
		setDebugMode(propertyConfigurator.getBoolean(PROPERTY_DEBUGMODE, false));
		setStartTLS(propertyConfigurator.getBoolean(PROPERTY_STARTTLS, false));		
			
	}	
	
	@Override
	public void start() throws Exception {
		super.start();
	}

	@Override
	public void stop() throws Exception {
		super.stop();		
	}
	
	@Override
	public void process(Exchange exchange) {
		synchronized (this) {
			log.debug("SPAGIC ["+Thread.currentThread().getId()+"] | Process In Only START");
			if (exchange.getStatus() == Status.Done) {
				return;
			} else if (exchange.getStatus() == Status.Error) {
				// Exchange has been aborted with an exception
				return;
			} else if (exchange.getFault() != null) {
				// Fault message
				exchange.setStatus(Status.Done);
				send(exchange);
			} else {
				Session session = null;
				try {
					Message in =  exchange.getIn(false);
					this.config.setStartTLS(startTLS);
					Properties props = MailUtils.getPropertiesForProtocol(
							this.config, this.customTrustManagers);
					props.put("mail.debug", isDebugMode() ? "true" : "false");

					// Get session
					session = Session.getInstance(props, config.getAuthenticator());

					// debug the session
					session.setDebug(this.debugMode);

					// get the transport from session
					Transport transport = session.getTransport(config.getProtocol());

					// Connect only once here
					// Transport.send() disconnects after each send
					// Usually, no username and password is required for SMTP
					transport.connect(config.getHost(), config.getPort(), config.getUsername(), config.getPassword());

					// Define message
					MimeMessage msg = new MimeMessage(session);
					
					
					// Set the subject, if provided
					if (getSubject() != null) {
						in.setHeader(AbstractMailMarshaler.MSG_TAG_SUBJECT, getSubject());
					}
					

					// let the marshaler to the conversion of message to mail
					this.marshaler.convertNormalizedMessageToMail(msg, exchange, in, this.sender, this.receiver);

					// Send message
					transport.sendMessage(msg, msg.getAllRecipients());

					// close transport
					transport.close();
					log.debug("SPAGIC ["+Thread.currentThread().getId()+"] | Process In Only END CORRECT ");
				} catch (MessagingException mex) {
					log.debug("SPAGIC ["+Thread.currentThread().getId()+"] | EXCEPTION ");
					log.error("Error sending mail...", mex);
				} finally {
					// delete all temporary allocated resources
					this.marshaler.cleanUpResources(exchange.getId());
				}
			}
		}
		
	}

	public AbstractMailMarshaler getMarshaler() {
		return marshaler;
	}

	public void setMarshaler(AbstractMailMarshaler marshaler) {
		this.marshaler = marshaler;
	}

	public String getCustomTrustManagers() {
		return customTrustManagers;
	}

	public void setCustomTrustManagers(String customTrustManagers) {
		this.customTrustManagers = customTrustManagers;
	}

	public MailConnectionConfiguration getConfig() {
		return config;
	}

	public void setConfig(MailConnectionConfiguration config) {
		this.config = config;
	}

	public String getConnection() {
		return connection;
	}

	public void setConnection(MailConnectionConfiguration config) {
//    	<protocol>://<user>@<host>[:<port>][/<folder>]?password=<password> 
//    	<protocol>://<host>[:<port>][/<folder>]?user=<user>;password=<password>
    	String connStr = config.getProtocol()+"://"+config.getHost();
    	connStr += config.getPort()!= -1 ? ":"+config.getPort() : "";
    	//connStr += config.getFolderName()!= "" ? "/"+config.getFolderName() : "";
    	connStr +="?user="+config.getUsername()+";password="+config.getPassword();

		this.connection = connStr;
	}

	public String getSender() {
		return sender;
	}

	public void setSender(String sender) {
		this.sender = sender;
	}

	public String getSubject() {
		return subject;
	}

	public void setSubject(String subject) {
		this.subject = subject;
	}

	public String getReceiver() {
		return receiver;
	}

	public void setReceiver(String receiver) {
		this.receiver = receiver;
	}

	public boolean isDebugMode() {
		return debugMode;
	}

	public void setDebugMode(boolean debugMode) {
		this.debugMode = debugMode;
	}

	public boolean isStartTLS() {
		return startTLS;
	}

	public void setStartTLS(boolean startTLS) {
		this.startTLS = startTLS;
	}

}
