/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.core;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Node;
import org.eclipse.ebpm.constants.SpagicConstants;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.messaging.api.Message;
import org.eclipse.ebpm.messaging.api.Pattern;
import org.eclipse.ebpm.messaging.api.Status;
import org.eclipse.ebpm.messaging.core.ExchangeImpl;
import org.eclipse.ebpm.messaging.core.MessageImpl;
import org.osgi.service.event.Event;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ExchangeUtils {
	
	public static Logger logger = LoggerFactory.getLogger(ExchangeUtils.class);
	public static SimpleDateFormat df = new SimpleDateFormat(SpagicConstants.SPAGIC_DATE_FORMAT);
	 /**
     * A helper method which will return true if the exchange is capable of both In and Out such as InOut,
     * InOptionalOut etc.
     *
     * @param exchange
     * @return true if the exchange can handle both input and output
     */
    public static boolean isInAndOut(Exchange exchange) {
    	return exchange.getPattern() == Pattern.InOut || exchange.getPattern() == Pattern.InOptionalOut;
    }
    
    public static boolean isInOnly(Exchange exchange) {
    	return exchange.getPattern() == Pattern.InOnly || exchange.getPattern() == Pattern.RobustInOnly;
    }
    
    public static Exchange createExchange(Pattern pattern) {
        return new ExchangeImpl(pattern);
    }
    
    public static Exchange createExchange(String id, Pattern pattern) {
        return new ExchangeImpl(id, pattern);
    }
    
    public static Exchange createExchange(String sender, String target, Pattern pattern ){
    	Exchange e = ExchangeUtils.createExchange(pattern);
		e.setProperty(SpagicConstants.SPAGIC_SENDER, sender);
		if (target != null)
			e.setProperty(SpagicConstants.SPAGIC_TARGET, target);
		return e;
	}
    
    public static  Exchange fromEvent(Event event){
    	String mep = (String)event.getProperty(SpagicConstants.EXCHANGE_MEP);
    	String exId = (String)event.getProperty(SpagicConstants.EXCHANGE_ID);
    	String status  = (String)event.getProperty(SpagicConstants.EXCHANGE_STATUS);
    	
    	
    	Exchange exchange = null;
    	if (mep.equalsIgnoreCase(SpagicConstants.IN_ONLY_MEP)){
    		exchange = ExchangeUtils.createExchange(exId, Pattern.InOnly);
    	}else if (mep.equalsIgnoreCase(SpagicConstants.IN_OUT_MEP)){
    		exchange = ExchangeUtils.createExchange(exId, Pattern.InOut);
    	}
    	
    	
    	String inBody = (String)event.getProperty(SpagicConstants.INBODY);
    	Message in = null;
    	Message out = null;
    	Message fault = null;
    	if (inBody != null){
    		in = exchange.getIn();
    		in.setBody(inBody);
    	}
    	String outBody = (String)event.getProperty(SpagicConstants.OUTBODY);
    	if (outBody != null){
    		out = exchange.getOut();
    		out.setBody(outBody);
    	}
    	String faultBody = (String)event.getProperty(SpagicConstants.FAULTBODY);
    	if (faultBody != null){
    		fault = exchange.getFault();
    		fault.setBody(faultBody);
    	}
    	
    	String[] pNames = event.getPropertyNames();
    	
    	String realPropertyName = null;
		for (int i = 0; i < pNames.length; i++) {
			if (pNames[i].startsWith(SpagicConstants.IN_PROPERTY)) {
				realPropertyName = pNames[i].substring(pNames[i].indexOf(".") + 1);
				if (in != null)
					in.setHeader(realPropertyName, event.getProperty(pNames[i]));
			} else if (pNames[i].startsWith(SpagicConstants.OUT_PROPERTY)) {
				realPropertyName = pNames[i].substring(pNames[i].indexOf(".") + 1 );
				if (out != null)
					out.setHeader(realPropertyName, event.getProperty(pNames[i]) );
			} else if (pNames[i].startsWith(SpagicConstants.FAULT_PROPERTY)) {
				realPropertyName = pNames[i].substring(pNames[i].indexOf(".") + 1);
				if (fault != null)
					fault.setHeader(realPropertyName, event.getProperty(pNames[i]));
			} else if (pNames[i].startsWith(SpagicConstants.EXCHANGE_PROPERTY)) {
				realPropertyName = pNames[i].substring(pNames[i].indexOf(".") + 1);
				
				if (!(realPropertyName.startsWith(SpagicConstants.WF_VARIABLE_PREFIX)))
					realPropertyName = pNames[i].substring(pNames[i].lastIndexOf(".") + 1 );
				
				if (exchange != null)
					exchange.setProperty(realPropertyName, event.getProperty(pNames[i]));
			} else if (pNames[i].equals(SpagicConstants.OUT_ATTACHMENT)) {
				if (out != null) {
					((MessageImpl)out)._decodeAttachments((String)event.getProperty(pNames[i]));
				}
			} else if (pNames[i].equals(SpagicConstants.IN_ATTACHMENT)) {
				((MessageImpl)in)._decodeAttachments((String)event.getProperty(pNames[i]));
			}

		}
    	if (in != null)
    		exchange.setIn(in);
    	if (out != null)
    		exchange.setOut(out);
    	if (fault != null)
    		exchange.setFault(fault);
    	
    	if (status.equalsIgnoreCase(SpagicConstants.STATUS_ACTIVE)){
    		exchange.setStatus(Status.Active);
    	}if (status.equalsIgnoreCase(SpagicConstants.STATUS_DONE)){
    		exchange.setStatus(Status.Done);
    	}if (status.equalsIgnoreCase(SpagicConstants.STATUS_ERROR)){
    		exchange.setStatus(Status.Error);
    	}
    	
    	if (exchange.getStatus().equals(Status.Error)){
    		String excMessage = (String)event.getProperty(SpagicConstants.EXCHANGE_ERROR);
    		exchange.setError(new Exception(excMessage));

    	}
    
    	return exchange;
    }
    
    public static String buildEventTopicFromExchange(Exchange exchange){
    		String target = (String)exchange.getProperties().get(SpagicConstants.SPAGIC_TARGET);
    		String targetNode = (String)exchange.getProperties().get(SpagicConstants.EXCHANGE_TARGET_NODE);
    		
    		String senderNode = (String)exchange.getProperties().get(SpagicConstants.EXCHANGE_SENDER_NODE);
    		exchange.getProperties().put(SpagicConstants.EXCHANGE_SENDER_NODE, SpagicNode.id());
    		if (senderNode != null){
    			targetNode = senderNode;
    		}
    		return SpagicNode.topicForServiceAndExternalNode(target, targetNode);
    		
    }
    
    
    public static Event toEvent(Exchange exchange){
    	String sender = (String)exchange.getProperties().get(SpagicConstants.SPAGIC_SENDER);
    	String target = (String)exchange.getProperties().get(SpagicConstants.SPAGIC_TARGET);
    	
    	
    	String eventTopic = buildEventTopicFromExchange(exchange);
    	
		
		Map properties = new HashMap();
		
		
		if (exchange.getPattern() == Pattern.InOnly)
			properties.put(SpagicConstants.EXCHANGE_MEP, SpagicConstants.IN_ONLY_MEP);
		else if ( exchange.getPattern() == Pattern.InOut){
			properties.put(SpagicConstants.EXCHANGE_MEP, SpagicConstants.IN_OUT_MEP);
		}
		properties.put(SpagicConstants.EXCHANGE_ID, exchange.getId());
		
		if (exchange.getStatus() == Status.Active){
			properties.put(SpagicConstants.EXCHANGE_STATUS, SpagicConstants.STATUS_ACTIVE);
		}else if (exchange.getStatus() == Status.Done){
			properties.put(SpagicConstants.EXCHANGE_STATUS, SpagicConstants.STATUS_DONE);
		}else if (exchange.getStatus() == Status.Error){
			properties.put(SpagicConstants.EXCHANGE_STATUS, SpagicConstants.STATUS_ERROR);
		}
		
		// Save exchangeProperties
		
		Map<String, Object> exchangeProperties = exchange.getProperties();
		
		String destPropertyName = null;
		
		for (String exProp : exchangeProperties.keySet()){
			destPropertyName = SpagicConstants.EXCHANGE_PROPERTY+"."+exProp;
			properties.put(destPropertyName, exchangeProperties.get(exProp));
		}
		
		Message in = exchange.getIn(false);
		if (in != null){
			properties.put(SpagicConstants.INBODY, in.getBodyText());
			
			Map<String, Object> inProperties = in.getHeaders();
			
			for (String inProperty : inProperties.keySet()){
				destPropertyName = SpagicConstants.IN_PROPERTY + "." + inProperty;
				properties.put(destPropertyName, inProperties.get(inProperty));
			}
			String encodedAtts = ((MessageImpl)in)._encodeAttachments();
			if (encodedAtts != null)
				properties.put(SpagicConstants.IN_ATTACHMENT, encodedAtts);
			
		}
		
		Message out = exchange.getOut(false);
		if (out != null){
			properties.put(SpagicConstants.OUTBODY, out.getBodyText());
			
			Map<String, Object> outProperties = out.getHeaders();
			
			for (String outProperty : outProperties.keySet()){
				destPropertyName = SpagicConstants.OUT_PROPERTY+ "." + outProperty;
				properties.put(destPropertyName, outProperties.get(outProperty));
			}
			String encodedAtts = ((MessageImpl)out)._encodeAttachments();
			if (encodedAtts != null)
				properties.put(SpagicConstants.OUT_ATTACHMENT, encodedAtts);			
		}
		
		Message fault = exchange.getFault(false);
		if (fault != null){
			properties.put(SpagicConstants.FAULTBODY, fault.getBodyText());
			
			Map<String, Object> faultProperties = fault.getHeaders();
			
			for (String faultProperty : faultProperties.keySet()){
				destPropertyName = SpagicConstants.FAULT_PROPERTY+ "." + faultProperty;
				properties.put(destPropertyName, faultProperties.get(faultProperty));
			}
		}
		
		if (exchange.getError() != null){
			properties.put(SpagicConstants.EXCHANGE_ERROR, exchange.getError().getMessage());
		}
		
		
		Event ev = new Event(eventTopic, properties);
		
		
		return ev;
    }
    
    public static boolean isSync(Exchange exchange){
    	Object syncExchange = (String)exchange.getProperty(SpagicConstants.SYNC_EXCHANGE);
    	
    	if (syncExchange != null)
    		return Boolean.valueOf((String)syncExchange);
    	return false;
    }
    
    
    public static void setSync(Exchange exchange){
    	exchange.setProperty(SpagicConstants.SYNC_EXCHANGE, true);
    }
    
    public static String headerAsXML(Message message){
    	
    	Map<String,Object> hdrs = message.getHeaders();
    	
    	if ((hdrs == null) || (hdrs.size() == 0))
    		return null;
    	Document doc = DocumentHelper.createDocument();
    	Element headersRoot = doc.addElement( "HEADERS" );
    	Iterator<String> it = hdrs.keySet().iterator();
    	while (it.hasNext()){
    		String id = it.next();
    		Object hdrValue = hdrs.get(id);
    		
    		if (hdrValue != null){
    			Element aHeader = headersRoot.addElement( "HEADER" ).addAttribute( "id",id  );
    			
    			fillHdrValue(aHeader, hdrValue);
    			
    		}
    	}
    	return doc.asXML();
    }
    
    public static void fillHdrValue(Element hdrElement, Object hdrValue){
    	if (hdrValue instanceof Map<?, ?>){
    		Map<String, ?> stringMap = (Map<String, ?>)hdrValue;
    		hdrElement.addAttribute("type", "Map");
    		
    		for (String key : stringMap.keySet() ){
    			Object elementMapValue = stringMap.get(key);
    			String stringValue = null;
    			if (elementMapValue instanceof String) {
    				stringValue = (String) elementMapValue;
    			} else if (elementMapValue instanceof org.w3c.dom.Node) {
    				stringValue = ((org.w3c.dom.Node) elementMapValue).getTextContent();
    			} else {
    				logger.error("Unsupported type in fillHdrValue. Type: " + elementMapValue.getClass().getName() + " for attribute: " + key);
    			}
    			
    			hdrElement.addElement("ENTRY").addAttribute("entryKey", key).addText(stringValue);
    		}
    		
    	}else if ( hdrValue instanceof Date ){
    		hdrElement.addAttribute("type", "Date");
    		hdrElement.setText(df.format((Date)hdrValue));
    	}else if ( hdrValue instanceof String ){
    		hdrElement.addAttribute("type", "String");
    		hdrElement.setText((String)hdrValue);
    	}else if ( hdrValue instanceof Boolean ){
    		hdrElement.addAttribute("type", "Boolean");
    		hdrElement.setText(String.valueOf(hdrValue));
    	}else if ( hdrValue instanceof Float ){
    		hdrElement.addAttribute("type", "Float");
    		hdrElement.setText(String.valueOf(hdrValue));
    	}else if ( hdrValue instanceof Double ){
    		hdrElement.addAttribute("type", "Double");
    		hdrElement.setText(String.valueOf(hdrValue));
    	}else if ( hdrValue instanceof Integer ){
    		hdrElement.addAttribute("type", "Integer");
    		hdrElement.setText(String.valueOf(hdrValue));
    	}
    }
    
    public static Object getHdrValue(Element hdrValueNode){
    	
    	String hdrType = hdrValueNode.attributeValue("type");
    	if (hdrType.equalsIgnoreCase("Map")){
    		Map<String, String> stringMap = new HashMap<String, String>();
    		List<Element> entries = hdrValueNode.elements("ENTRY");
    		
    		for (Element entry : entries ){
    			stringMap.put(entry.attribute("entryKey").getValue(), entry.getText());
    		}
    		return stringMap;
    	}else if (hdrType.equalsIgnoreCase("Date")){
    		try{
    			return df.parse(hdrValueNode.getText());
    		}catch (Exception e) {
				return null;
			}
    	}else if (hdrType.equalsIgnoreCase("String")){
    		return hdrValueNode.getText();
    	}else if (hdrType.equalsIgnoreCase("Boolean")){
    		return Boolean.valueOf(hdrValueNode.getText());
    	}else if (hdrType.equalsIgnoreCase("Float")){
    		return Float.valueOf(hdrValueNode.getText());
    	}else if (hdrType.equalsIgnoreCase("Double")){
    		return Double.valueOf(hdrValueNode.getText());
    	}else if (hdrType.equalsIgnoreCase("Integer")){
    		return Integer.valueOf(hdrValueNode.getText());
    	}else{
    		return hdrValueNode.getText();
    	}
    }
  
    
    public static void fillHeaders(Message message, String headersAsXML){
    	
    	if (( headersAsXML == null ) || ( headersAsXML.trim().length() == 0))
    		return;
    	
    	try{
    		Document doc = DocumentHelper.parseText(headersAsXML);
    		Map<String,Object> hdrs = new HashMap<String, Object>();
        	
    		
    		List<Element> nl = doc.getRootElement().elements("HEADER");
    		
    		for ( Element n : nl){
    			String id = n.attributeValue("id");
    			Object objValue = getHdrValue(n);
    			hdrs.put(id, objValue);
    		}
        	
        	message.setHeaders(hdrs);
    	}catch (Exception e) {
			// TODO: handle exception
		}
    	
    	return;
    }
}
