/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.core.routing;

import java.util.UUID;
import java.util.concurrent.atomic.AtomicReference;

import org.eclipse.ebpm.constants.SpagicConstants;
import org.eclipse.ebpm.core.ExchangeUtils;
import org.eclipse.ebpm.core.SpagicNode;
import org.eclipse.ebpm.core.registry.ISpagicRegistry;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.messaging.api.Status;
import org.osgi.framework.ServiceReference;
import org.osgi.service.event.EventAdmin;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class SpagicRouter implements IMessageRouter {
	
	private static Logger logger = LoggerFactory.getLogger(SpagicRouter.class);
	
	private AtomicReference<EventAdmin> localEventAdmin = new AtomicReference<EventAdmin>();
	private AtomicReference<EventAdmin> distributedEventAdmin = new AtomicReference<EventAdmin>();
	
	private ISpagicRegistry registry = null;
	private IDynamicRouter dynamicRouter = null;
	
	
	
	
	public void send(Exchange exchange) throws Exception {
		
		String sender = (String)exchange.getProperties().get(SpagicConstants.SPAGIC_SENDER);
		
		// Target Configuration
		String target = (String)exchange.getProperties().get(SpagicConstants.SPAGIC_TARGET);
		
		logger.info("Spagic Router SEND EXCHANGE ["+exchange.getId()+"] ["+sender+"] ["+target+"] Status ["+exchange.getStatus().toString()+"]");

        String correlationId = (String)exchange.getProperty(SpagicConstants.CORRELATION_ID);
        if (correlationId == null) {
        	correlationId = UUID.randomUUID().toString();
        	exchange.setProperty(SpagicConstants.CORRELATION_ID, correlationId);
        }

		// If target is not configured as a properties of the exchange
		// Se if there's a connector routing map
		if (target == null){
			target = dynamicRouter.getTarget(sender);
			exchange.setProperty(SpagicConstants.SPAGIC_TARGET, target);
		}
		
		if (target == null)
			throw new Exception(" No Target Configured ");
		
		if (!registry.isRegistered(target)){
			String error = "Target Service ["+target+"] for exchange is not registered";
			sendBack(exchange, error, sender, target);
		}
		
		route(exchange, target);
	}
	
	protected String getSenderNode(Exchange exchange){
		String senderNode = (String)exchange.getProperties().get(SpagicConstants.EXCHANGE_SENDER_NODE);
		return senderNode;
	}
	
	protected String getTargetNode(Exchange exchange){
		String targetNode = (String)exchange.getProperties().get(SpagicConstants.EXCHANGE_TARGET_NODE);
		return targetNode;
	}
		
	protected void route(Exchange exchange, String target){
		logger.debug("SpagicRouter -> route -> Target is -> ["+target+"]");
		
		
		EventAdmin ea = null;
		if (registry.isRegisteredLocally(target)){
			
			logger.debug("SpagicRouter-> route -> Target ["+target+"] is registered locally ");
			logger.debug("SpagicRouter -> route -> Setting Target Node -> ["+SpagicNode.id()+"]");
			exchange.setProperty(SpagicConstants.EXCHANGE_TARGET_NODE, SpagicNode.id());
			logger.debug("SpagicRouter -> route -> Getting Local Event Admin");
			ea = localEventAdmin.get();
			if (ea == null){
				logger.debug("SpagicRouter -> route -> Local Event Admin is null -> Getting Distributed Event Admin");
				ea = distributedEventAdmin.get();
			}
		}else if (registry.isRegistered(target)){
			
			String node = registry.getServiceNode(target);
			logger.debug("SpagicRouter-> route -> Target ["+target+"] is NOT registered locally BUT on Node ["+node+"]");
			logger.debug("SpagicRouter -> route -> Setting Target Node -> ["+node+"]");
			exchange.setProperty(SpagicConstants.EXCHANGE_TARGET_NODE, node);
			logger.debug("SpagicRouter -> route -> Getting Distributed Event Admin");
			ea = distributedEventAdmin.get();
			if (ea == null){
				logger.debug("SpagicRouter -> route -> Distributed Event Admin is null -> Getting Local Event Admin");
				ea = localEventAdmin.get();
			}
		}
		
		logger.debug(" Checking if exchange has a sender Node and if it's different from target Node");
		String senderNode = getSenderNode(exchange);
		String targetNode = getTargetNode(exchange);
		logger.debug(" Sender Node ["+senderNode+"] -- Target Node ["+targetNode+"]");
		if ((senderNode != null) && !(senderNode.equalsIgnoreCase(targetNode))){
			logger.debug("SpagicRouter -> Forced to use Distributed Event Admin");
			exchange.setProperty(SpagicConstants.EXCHANGE_TARGET_NODE, senderNode);
			ea = distributedEventAdmin.get();
			if (ea == null){
				ea = distributedEventAdmin.get();
			}
		}
		
		if (ea != null){
			ea.postEvent(ExchangeUtils.toEvent(exchange));
		}
	}
	

	protected void sendBack(Exchange exchange, String errorMessage, String originalSender, String originalTarget){
		if (!(exchange.getStatus() == Status.Done)){
			exchange.setProperty(SpagicConstants.SPAGIC_SENDER, SpagicConstants.SPAGIC_ROUTER);
			exchange.setProperty(SpagicConstants.SPAGIC_TARGET, originalSender);
			if (ExchangeUtils.isInAndOut(exchange)){
				exchange.getFault().setBody("<error>"+errorMessage+"</error>");
			}else{
				exchange.setError(new Exception(errorMessage));
			}
			
			route(exchange, originalSender);
			return;
		}
	}
	
	
	public void bindLocalEventAdmin(EventAdmin ea) {
		localEventAdmin.set(ea);
	}
	
	public void unbindLocalEventAdmin(EventAdmin ea) {
		localEventAdmin.compareAndSet(ea, null);
	}
	
	
	public void bindDistributedEventAdmin(EventAdmin ea) {
		distributedEventAdmin.set(ea);
	}
	
	public void unbindDistributedEventAdmin(EventAdmin ea) {
		distributedEventAdmin.compareAndSet(ea, null);
	}

	public boolean isDistributedEventAdmin(ServiceReference sr){
		String eaType = (String) sr.getProperty(SpagicConstants.EVENT_ADMIN_TYPE);
		if (eaType == null){
			return false;
		}else{
			return eaType.equalsIgnoreCase(SpagicConstants.DISTRIBUTED_EVENT_ADMIN_TYPE);
		}
	}
	

	

			
	
	
	
	
	
	public void unbindRegistry(ISpagicRegistry spagicRegistry){
		this.registry = null;
	}
	
	public void bindRegistry(ISpagicRegistry spagicRegistry){
		this.registry = spagicRegistry;
	}

	
	
	public void bindDynamicRouter(IDynamicRouter dr){
		this.dynamicRouter = dr;
	}

	public void unbindDynamicRouter(IDynamicRouter dr){
		this.dynamicRouter = null;
	}


	public boolean isServiceRegistered(String target) throws Exception {
		// TODO Auto-generated method stub
		return registry.isRegistered(target);
	}
}
