/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.dirwatcher;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.ConcurrentHashMap;

import org.dom4j.Attribute;
import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.eclipse.ebpm.constants.SpagicConstants;
import org.eclipse.ebpm.deployer.IDeploymentService;
import org.osgi.service.component.ComponentContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class DirWatcherService {
	
	protected Logger logger = LoggerFactory.getLogger(DirWatcherService.class); 
	private ConcurrentHashMap<String, String> servicesFileToIdMap = new ConcurrentHashMap<String, String>();
	private ConcurrentHashMap<String, String> connectorsFileToIdMap = new ConcurrentHashMap<String, String>();
	private ConcurrentHashMap<String, String> datasourcesFileToIdMap = new ConcurrentHashMap<String, String>();
	private ConcurrentHashMap<String, String> connectionFactoriesFileToIdMap = new ConcurrentHashMap<String, String>();
	private ConcurrentHashMap<String, List<String>> routesForFileMap = new ConcurrentHashMap<String, List<String>>();
	
	
	protected void activate(ComponentContext componentContext){
		final IDeploymentService deploymentService = (IDeploymentService)componentContext.locateService("deploymentService");
	
		String spagicHome = System.getProperty("spagic.home");
		
		if (spagicHome == null){
			logger.warn(" Spagic HOME NOT SETTED ");
			return;
		}
			
		String servicesHome = spagicHome + File.separator + SpagicConstants.SERVICES_FOLDER;
		String connectorsHome = spagicHome + File.separator + SpagicConstants.CONNECTORS_FOLDER;
		String dsHome = spagicHome + File.separator + SpagicConstants.DATASOURCES_FOLDER;
		String cfHome = spagicHome + File.separator + SpagicConstants.CONNECTIONFACTORIES_FOLDER;
		String routesHome = spagicHome + File.separator + SpagicConstants.ROUTES_FOLDER;
		
		TimerTask servicePollingTask = new DirWatcher(servicesHome, SpagicConstants.SERVICE_DEPLOYMENTS_EXTENSIONS ) {
			protected void onChange(File file, String action) {
				try{
					String filePath = file.getCanonicalPath();
					logger.info("DirWatcher Service File ["+filePath+"] --> Action ["+action+"]");
			
						if (action.equalsIgnoreCase("delete")){
							
							String spagicId = servicesFileToIdMap.get(filePath);
							
							if (spagicId != null){
								logger.warn("DirWatcher -> No component deployed for this file");
								deploymentService.undeployService(spagicId);
							}
							servicesFileToIdMap.remove(filePath);
							
						}else if (action.equalsIgnoreCase("add")){
							SAXReader reader = new SAXReader();
							Document doc = reader.read(file);
							
							GenericDeploymentDescriptor cd = new GenericDeploymentDescriptor(doc);
							
							deploymentService.deployService(cd.getSpagicId(), 
															cd.getFactoryName(), 
															cd.getProperties());
							
							servicesFileToIdMap.put(filePath, cd.getSpagicId());
							
						}else if (action.equalsIgnoreCase("modify")){
							SAXReader reader = new SAXReader();
							Document doc = reader.read(file);
							
							GenericDeploymentDescriptor cd = new GenericDeploymentDescriptor(doc);
							
							deploymentService.undeployService(cd.getSpagicId());
							
							deploymentService.deployService(cd.getSpagicId(), 
									cd.getFactoryName(), 
									cd.getProperties());
						}else{
							logger.warn("DirWatcher Service action ["+action+"] unknown");
						}
				}catch(Exception e){
					logger.error(e.getMessage(), e);
				}		
			}
		};

		
		TimerTask connectorsPollingTask = new DirWatcher(connectorsHome, SpagicConstants.CONNECTOR_DEPLOYMENTS_EXTENSIONS) {
			protected void onChange(File file, String action) {
				try{
					String filePath = file.getCanonicalPath();
					logger.info("DirWatcher Connector File ["+filePath+"] --> Action ["+action+"]");
			
						if (action.equalsIgnoreCase("delete")){
							
							String spagicId = connectorsFileToIdMap.get(filePath);
							
							if (spagicId != null){
								logger.warn("DirWatcher -> No component deployed for this file");
								deploymentService.undeployConnector(spagicId);
							}
							connectorsFileToIdMap.remove(filePath);
							
						}else if (action.equalsIgnoreCase("add")){
							SAXReader reader = new SAXReader();
							Document doc = reader.read(file);
							
							GenericDeploymentDescriptor cd = new GenericDeploymentDescriptor(doc);
							
							deploymentService.deployConnector(cd.getSpagicId(), 
															cd.getFactoryName(), 
															cd.getProperties());
							
							connectorsFileToIdMap.put(filePath, cd.getSpagicId());
							
						}else if (action.equalsIgnoreCase("modify")){
							SAXReader reader = new SAXReader();
							Document doc = reader.read(file);
							
							GenericDeploymentDescriptor cd = new GenericDeploymentDescriptor(doc);
						
							deploymentService.undeployConnector(cd.getSpagicId());
							
							deploymentService.deployConnector(cd.getSpagicId(), 
									cd.getFactoryName(), 
									cd.getProperties());
						}else{
							logger.warn("DirWatcher Service action ["+action+"] unknown");
						}
				}catch(Exception e){
					logger.error(e.getMessage(), e);
				}		
			}
		};
		
		
		TimerTask dataSourcesPollingTask = new DirWatcher(dsHome, SpagicConstants.DATASOURCE_DEPLOYMENTS_EXTENSIONS) {
			protected void onChange(File file, String action) {
				try{
					String filePath = file.getCanonicalPath();
					logger.info("DirWatcher Datasource File ["+filePath+"] --> Action ["+action+"]");
			
						if (action.equalsIgnoreCase("delete")){
							
							String dataSourceId = datasourcesFileToIdMap.get(filePath);
							
							if (dataSourceId != null){
								logger.warn("DirWatcher -> No component deployed for this file");
								deploymentService.undeployDatasource(dataSourceId);
							}
							datasourcesFileToIdMap.remove(filePath);
							
						}else if (action.equalsIgnoreCase("add")){
							
							SAXReader reader = new SAXReader();
							Document doc = reader.read(file);
						
							GenericDeploymentDescriptor dsDesc = new GenericDeploymentDescriptor(doc);
						
							deploymentService.deployDatasource(dsDesc.getId(), dsDesc.getProperties());
							datasourcesFileToIdMap.put(filePath, dsDesc.getId());
					
							
						}else if (action.equalsIgnoreCase("modify")){
							logger.warn("DirWatcher DataSource action ["+action+"] not supported");
						}else{
							logger.warn("DirWatcher DataSource action ["+action+"] unknown");
						}
				}catch(Exception e){
					logger.error(e.getMessage(), e);
				}		
			}
		};
		
		TimerTask connectionFactoriesPollingTask = new DirWatcher(cfHome, SpagicConstants.CONNECTIONFACTORY_DEPLOYMENTS_EXTENSIONS) {
			protected void onChange(File file, String action) {
				try{
					String filePath = file.getCanonicalPath();
					logger.info("DirWatcher JMS ConnectionFactory File ["+filePath+"] --> Action ["+action+"]");
			
					if (action.equalsIgnoreCase("delete")){
						String connectionFactoryId = connectionFactoriesFileToIdMap.get(filePath);
						
						if (connectionFactoryId != null){
							logger.warn("DirWatcher -> No component deployed for this file");
							deploymentService.undeployConnectionFactory(connectionFactoryId);
						}
						connectionFactoriesFileToIdMap.remove(filePath);
						
					}else if (action.equalsIgnoreCase("add")){
						
						SAXReader reader = new SAXReader();
						Document doc = reader.read(file);
						
						GenericDeploymentDescriptor depDesc = new GenericDeploymentDescriptor(doc);

						
						
						deploymentService.deployConnectionFactory(depDesc.getId(), depDesc.getFactoryName(), depDesc.getProperties());
						connectionFactoriesFileToIdMap.put(filePath, depDesc.getId());				
						
					}else if (action.equalsIgnoreCase("modify")){
						logger.warn("DirWatcher ConnectionFactory action ["+action+"] not supported");
					}else{
						logger.warn("DirWatcher ConnectionFactory action ["+action+"] unknown");
					}
				}catch(Exception e){
					logger.error(e.getMessage(), e);
				}		
			}
		};

		TimerTask routesPollingTask = new DirWatcher(routesHome, SpagicConstants.ROUTES_DEPLOYMENTS_EXTENSIONS ) {
			protected void onChange(File file, String action) {
				try{
					String filePath = file.getCanonicalPath();
					logger.info("DirWatcher Routes File ["+filePath+"] --> Action ["+action+"]");
			
						if (action.equalsIgnoreCase("delete")){
							
							List<String> routesForFile = routesForFileMap.get(filePath);
							deploymentService.deployRoutes(null, routesForFile);
							routesForFileMap.remove(filePath);
							
						}else {
							SAXReader reader = new SAXReader();
							Document doc = reader.read(file);
							
							Element routesElement = doc.getRootElement();
							List<Element> routesNode  = routesElement.elements("spagic:route");
							
							
							List<String> updateRoutesForFile = new ArrayList<String>();
							
							Attribute fromAttribute = null;
							Attribute toAttribute = null;
							
							for (Element pn : routesNode){
								fromAttribute = pn.attribute("from");
								toAttribute = pn.attribute("to");
								
								updateRoutesForFile.add(fromAttribute.getValue()+";"+ toAttribute.getValue());
							}
							if (action.equalsIgnoreCase("add")){
								deploymentService.deployRoutes(updateRoutesForFile, null);
							} else if (action.equalsIgnoreCase("modify")){
								List<String> oldRoutes = routesForFileMap.get(filePath);
								deploymentService.deployRoutes(updateRoutesForFile, oldRoutes);
							}
							routesForFileMap.put(filePath, updateRoutesForFile);
							
							logger.warn("DirWatcher Service action ["+action+"] ");
						}
				}catch(Exception e){
					logger.error(e.getMessage(), e);
				}		
			}
		};
		
		Timer timerService = new Timer();
		timerService.schedule(servicePollingTask, new Date(), 5000);
		
		Timer timerConnectors = new Timer();
		timerConnectors.schedule(connectorsPollingTask, new Date(), 5000);
		
		Timer timerDataSources = new Timer();
		timerDataSources.schedule(dataSourcesPollingTask, new Date(), 5000);
		
		Timer timerConnectionFactories = new Timer();
		timerConnectionFactories.schedule(connectionFactoriesPollingTask, new Date(), 5000);	
		
		Timer timerRouterService = new Timer();
		timerRouterService.schedule(routesPollingTask, new Date(), 5000);
	}
	
	protected void deactivate(ComponentContext componentContext){
	
	}
	
}
