/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.distributed;
import java.util.Properties;

import org.eclipse.ebpm.constants.SpagicConstants;
import org.eclipse.ecf.core.ContainerConnectException;
import org.eclipse.ecf.core.ContainerCreateException;
import org.eclipse.ecf.core.IContainer;
import org.eclipse.ecf.core.IContainerFactory;
import org.eclipse.ecf.core.IContainerManager;
import org.eclipse.ecf.core.identity.IDFactory;
import org.eclipse.ecf.core.sharedobject.ISharedObjectContainer;
import org.eclipse.ecf.core.sharedobject.SharedObjectAddException;
import org.eclipse.ecf.remoteservice.eventadmin.DistributedEventAdmin;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;
import org.osgi.service.event.EventConstants;
import org.osgi.util.tracker.ServiceTracker;

public class DistributedEventAdminActivator implements BundleActivator {

	public static final String DEFAULT_TOPIC = "defaultTopic";

	protected BundleContext bundleContext;

	// The following must be set in processArgs
	protected String containerType;
	protected String containerId;
	protected String targetId;
	protected String topic = DEFAULT_TOPIC;

	protected ServiceTracker containerManagerTracker;
	
	protected DistributedEventAdmin eventAdminImpl;
	protected ServiceRegistration eventAdminRegistration;
	protected IContainer container;

	

	private static final String ECF_MANAGER_CONTAINER_TYPE = "ecf.jms.activemq.tcp.manager";
	public static final String ECF_MANAGER_CONTAINER_ID = "tcp://localhost:61616/exampleTopic";
	
	private static final String DEFAULT_CONTAINER_TYPE = "ecf.jms.activemq.tcp.client";
	

	
	@Override
	public void start(BundleContext context) throws Exception {
		// Get BundleContext
		
		String isDistributedStr = System.getProperty(SpagicConstants._IS_CLUSTER_ENABLED);
		boolean isDistributed = ( ( isDistributedStr != null ) ? Boolean.valueOf(isDistributedStr) : false);
		
		if (!isDistributed)
			return;
		
		bundleContext =context;

		containerId = ECF_MANAGER_CONTAINER_ID;
		containerId = System.getProperty(SpagicConstants.DISTRIBUTED_EVENT_ADMIN_TOPIC_URL);
		containerType = ECF_MANAGER_CONTAINER_TYPE;
		
		
		//
		// Create event admin impl
		//
		eventAdminImpl = new DistributedEventAdmin(bundleContext);

		try{
			containerId = ECF_MANAGER_CONTAINER_ID;
			containerType = ECF_MANAGER_CONTAINER_TYPE;
			if (System.getProperty(SpagicConstants.DISTRIBUTED_EVENT_ADMIN_TOPIC_URL) != null){
				containerId = System.getProperty(SpagicConstants.DISTRIBUTED_EVENT_ADMIN_TOPIC_URL);
			}
			createConfigureAndConnectContainer();
		}catch (Throwable t) {
			
			containerId = null;
			containerType = DEFAULT_CONTAINER_TYPE;
			targetId = ECF_MANAGER_CONTAINER_ID;
			if (System.getProperty(SpagicConstants.DISTRIBUTED_EVENT_ADMIN_TOPIC_URL) != null){
				targetId = System.getProperty(SpagicConstants.DISTRIBUTED_EVENT_ADMIN_TOPIC_URL);
			}
			topic = DEFAULT_TOPIC;
			createConfigureAndConnectContainer();
		}
			
		
		// Create, configure, and connect container
		

		// start event admin
		eventAdminImpl.start();
		// register as EventAdmin service instance
		Properties props = new Properties();
		props.put(EventConstants.EVENT_TOPIC, topic);
		props.put(SpagicConstants.EVENT_ADMIN_TYPE, SpagicConstants.DISTRIBUTED_EVENT_ADMIN_TYPE);
		
		eventAdminRegistration = context.registerService(
				"org.osgi.service.event.EventAdmin", eventAdminImpl, props);

	}

	@Override
	public void stop(BundleContext context) throws Exception {
		if (eventAdminRegistration != null) {
			eventAdminRegistration.unregister();
			eventAdminRegistration = null;
		}
		if (container != null) {
			container.dispose();
			getContainerManager().removeAllContainers();
			container = null;
		}
		if (containerManagerTracker != null) {
			containerManagerTracker.close();
			containerManagerTracker = null;
		}

	}

	protected boolean isThereEventAdminManager(){
		IContainer[] containers = getContainerManager().getAllContainers();
		
		for ( int i=0; i< containers.length; i++){
			ISharedObjectContainer soContainer = (ISharedObjectContainer) containers[i].getAdapter(ISharedObjectContainer.class);
			if ((soContainer != null) && soContainer.isGroupManager())
					return true;
		}
		
		return false;
	}
	protected void createConfigureAndConnectContainer()
			throws ContainerCreateException, SharedObjectAddException,
			ContainerConnectException {
		// get container factory and create container
		IContainerFactory containerFactory = getContainerManager().getContainerFactory();
	
		
		container = (containerId == null) ? containerFactory.createContainer(containerType) : containerFactory
				.createContainer(containerType, new Object[] { containerId });

		// Get socontainer
		ISharedObjectContainer soContainer = (ISharedObjectContainer) container.getAdapter(ISharedObjectContainer.class);
		
		// Add to soContainer, with topic as name
		soContainer.getSharedObjectManager().addSharedObject(IDFactory.getDefault().createStringID(topic), eventAdminImpl,null);

		if ( soContainer.isGroupManager() ){
			System.out.println(" Event Admin " + soContainer);
		}
		// then connect to target Id
		if (targetId != null)
			container.connect(IDFactory.getDefault().createID(
					container.getConnectNamespace(), targetId), null);
	}

	protected IContainerManager getContainerManager() {
		if (containerManagerTracker == null) {
			containerManagerTracker = new ServiceTracker(bundleContext,
					IContainerManager.class.getName(), null);
			containerManagerTracker.open();
		}
		return (IContainerManager) containerManagerTracker.getService();
	}

}
