/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.messaging.core;

import java.io.IOException;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.activation.DataHandler;
import javax.security.auth.Subject;
import javax.xml.transform.Source;

import org.eclipse.ebpm.attachments.Attachment;
import org.eclipse.ebpm.attachments.AttachmentFacade;
import org.eclipse.ebpm.attachments.AttachmentUtils;
import org.eclipse.ebpm.messaging.api.Message;
import org.eclipse.ebpm.messaging.core.util.SourceTransformer;
import org.eclipse.ebpm.messaging.core.util.StringSource;


public class MessageImpl implements Message {

    /**
     * Generated serial version UID
     */
    private static final long serialVersionUID = -8621182821298293687L;

    private Source body;
    private String contentType;
    private String contentEncoding;
    private Subject securitySubject;
    private Map<String, Object> headers;
    
    // ID_IN THE MESSAGE, ID IN PERSITENT STORE
    private Map<String, String> _internalAttachments;
  

    public MessageImpl() {
    }

    /**
     * Returns the body of the message in its default format.
     */
    public Source getBody() {
        return body;
    }

   

    /**
     * Set the body of the message.
     *
     * @param body the body of the message
     */
    public void setBody(Source body) {
        this.body = body;
    }

   

    /**
     * Get the mime body type describing the body of the message
     *
     * @return the mime body type
     */
    public String getContentType() {
        return contentType;
    }

    /**
     * Set the mime body type describing the body of the message
     *
     * @param type the mime body type
     */
    public void setContentType(String type) {
        this.contentType = type;
    }

    /**
     * Get the encoding of the message
     *
     * @return the encoding
     */
    public String getContentEncoding() {
        return contentEncoding;
    }

    /**
     * Set the encoding of the message
     *
     * @param encoding the encoding
     */
    public void setContentEncoding(String encoding) {
        this.contentEncoding = encoding;
    }

    /**
     * Get the security subject for this message
     *
     * @return the security subject
     */
    public Subject getSecuritySubject() {
        return securitySubject;
    }

    /**
     * Set the security subject for this message
     *
     * @param securitySubject the security subject
     */
    public void setSecuritySubject(Subject securitySubject) {
        this.securitySubject = securitySubject;
    }

    /**
     * Get a header on this message.
     *
     * @param name the name of the header
     * @return the value of the header of <code>null</code> if none has been set
     */
    public Object getHeader(String name) {
        if (headers == null) {
            return null;
        }
        return headers.get(name);
    }

  

    public void checkHeaderType(Object object){
    	if (! ( (object instanceof String) || (object instanceof Date) 
    		 || (object instanceof Integer) || (object instanceof Boolean)
    	     || (object instanceof Float)|| (object instanceof Double))){
    	
    		if (object instanceof Map<?, ?>){
    			try{
    				Map<String, String> stringMap = (Map<String, String>)object;
    			}catch (Exception e) {
    				 throw new RuntimeException("Only Map<String,String>  are supported header types");
				}
    		}else{
    			 throw new RuntimeException("Only Date, Integer, Float, Double, String, Boolean are supported header types");
    		}
    	}
	     
	    
		
    	
    }

    /**
     * Set a header for this message
     *
     * @param name  the name of the header
     * @param value the value of the header
     */
    public void setHeader(String name, Object value) {
    	checkHeaderType(value);
        if (headers == null) {
            headers = new HashMap<String, Object>();
        }
        headers.put(name, value);
    }

    

    /**
     * Remove the given header and returns its value.
     *
     * @param name the name of the header
     * @return the previous value
     */
    public Object removeHeader(String name) {
        if (headers == null) {
            return null;
        }
        return headers.remove(name);
    }

   

    /**
     * Get a map of all the headers for this message
     *
     * @return a map of headers
     */
    public Map<String, Object> getHeaders() {
        if (headers == null) {
            headers = new HashMap<String, Object>();
        }
        return headers;
    }

    /**
     * Set all the headers
     *
     * @param headers the new map of headers
     */
    public void setHeaders(Map<String, Object> headers) {
        this.headers = headers;
    }

    /**
     * Retrieve an attachment given its id.
     *
     * @param id the id of the attachment to retrieve
     * @return the attachement or <code>null</code> if none exists
     */
    public Object getAttachment(String id) {
        if (_internalAttachments == null) {
            return null;
        }
        
        String idInPersistentStore = _internalAttachments.get(id);
        Attachment att = AttachmentFacade.load(idInPersistentStore);
        return att.getDh();
    }

    /**
     * Add an attachment to this message
     *
     * @param id    the id of the attachment
     * @param value the attachment to add
     */
    public void addAttachment(String id, Object value) {
    	if (_internalAttachments == null) {	  
    		  _internalAttachments = new HashMap<String, String>();
    		 
        }
    	String idInPersistenceStore = AttachmentFacade.store(id, (DataHandler)value);
       _internalAttachments.put(id, idInPersistenceStore);
    }

    /**
     * Remove an attachment on this message
     *
     * @param id the id of the attachment to remove
     */
    public void removeAttachment(String id) {
        if (_internalAttachments != null) {
            String idInPersistenceStore = _internalAttachments.get(id);
            AttachmentFacade.delete(idInPersistenceStore);
            _internalAttachments.remove(id);
        }
    }

    /**
     * Retrieve a map of all attachments
     *
     * @return the map of attachments
     */
    public Map<String, Object> getAttachments() {
        if (_internalAttachments == null) {
        	_internalAttachments = new HashMap<String, String>();
        }
        Map<String, Object> realAttachmentMap = new HashMap<String, Object>();
        for (String idAtt :_internalAttachments.keySet()){
        	realAttachmentMap.put(idAtt, getAttachment(idAtt));
        }
        return realAttachmentMap;
    }

  
    

    /**
     * Copies the contents of the other message into this message
     *
     * @param msg the message to copy from
     */
    public void copyFrom(Message msg) {
        body = msg.getBody();
        if (!msg.getHeaders().isEmpty()) {
            headers = new HashMap<String, Object>();
            for (Map.Entry<String, Object> e : msg.getHeaders().entrySet()) {
                headers.put(e.getKey(), e.getValue());
            }
        } else {
            headers = null;
        }
        if (!msg.getAttachments().isEmpty()) {
            
            for (Map.Entry<String, Object> e : msg.getAttachments().entrySet()) {
                addAttachment(e.getKey(), e.getValue());
            }
        } else {
            //attachments = null;
        }
    }

    /**
     * Creates a copy of this message so that it can
     * be used and possibly modified further in another exchange
     *
     * @return a new message instance copied from this message
     */
    public Message copy() {
        MessageImpl copy = new MessageImpl();
        copy.copyFrom(this);
        return copy;
    }

    private void readObject(java.io.ObjectInputStream stream) throws IOException, ClassNotFoundException {
        stream.defaultReadObject();
    }

    private void writeObject(java.io.ObjectOutputStream out) throws IOException {

        out.defaultWriteObject();
    }

   

   public String display(boolean displayContent) {
        return "Message []";
    }

    public String toString() {
        return display(false);
    }
    
    public String getBodyText()  {
    	try{
    		 return new SourceTransformer().toString(body);
    	}catch (Exception e) {
			throw new RuntimeException(e);
		}
       
    }

    public void setBodyText(String xml) {
        setBody(new StringSource(xml));
    }
    
    public void setBody(String xml) {
        setBody(new StringSource(xml));
    }

    public String _encodeAttachments(){
    	if ((_internalAttachments == null) || (_internalAttachments.size() == 0))
    		return null;
    	StringBuffer sb = new StringBuffer();
    	Iterator<String> it = _internalAttachments.keySet().iterator();
    	while (it.hasNext()){
    		String id = it.next();
    		sb.append(id +","+_internalAttachments.get(id));
    		if (it.hasNext())
    			sb.append(";");
    	}
    	return sb.toString();
    }
    
    public void _decodeAttachments(String encodedAttachments){
    	if (encodedAttachments == null)
    		return;
    	
    	String[] attachmentTokens = AttachmentUtils.decodeAttachmentsIds(encodedAttachments);
    	HashMap<String, String> internalAttachments = AttachmentUtils.decodeAttachmentsIds(attachmentTokens);
    	if (_internalAttachments == null)
    		_internalAttachments = internalAttachments;
    	else
    		_internalAttachments.putAll(internalAttachments);
    }
}
