/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.eclipse.ebpm.monitoring;

import java.util.Date;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.atomic.AtomicReference;

import org.eclipse.ebpm.constants.SpagicConstants;
import org.eclipse.ebpm.databasemanager.IDatabaseManager;
import org.eclipse.ebpm.metadb.constants.ServiceInstanceStateConstants;
import org.eclipse.ebpm.metadb.model.Service;
import org.eclipse.ebpm.metadb.model.ServiceInstance;
import org.osgi.service.event.Event;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class MonitorEventConsumer extends Thread {
	 
	 private static Logger logger = LoggerFactory.getLogger(MonitorService.class);
	 private BlockingQueue<Event> blockingQueue;
	 private String consumerName;
	 private final AtomicReference<IDatabaseManager> dbManagerAR = new AtomicReference<IDatabaseManager>(null);
	 
	 public MonitorEventConsumer(String consumerName, BlockingQueue<Event> blockingQueue, IDatabaseManager dbManager){
			    this.consumerName = consumerName;
			    this.blockingQueue = blockingQueue;
			    dbManagerAR.set(dbManager);
	 }

	public void run() {
		while (true) {
			try {
				Event ev = blockingQueue.take();
				consumeEvent(ev);
			} catch (InterruptedException ie) {
				logger.error("Interrupted", ie);
			}
		}
	}
	
	public void consumeEvent(Event event) {
		
		try {
			for (String propertyName : event.getPropertyNames())
				logger.debug(propertyName + " = " + event.getProperty(propertyName));
			
			IDatabaseManager dbManager = getDatabaseManager();
			Boolean internalEvent = (Boolean) event.getProperty(SpagicConstants._IS_INTERNAL_EVENT);
			
			if ((internalEvent != null) && internalEvent) {
				
				// Check if the internal event is of interest for the monitoring
				if (event.getProperty(SpagicConstants._INTERNAL_EVENT_TYPE).equals(SpagicConstants._INTERNAL_EVENT_PROCESS_STARTED)) {
					String exchangeID = (String) event.getProperty(SpagicConstants._INTERNAL_EVENT_REFERRING_ID);
					Long processId = (Long) event.getProperty(SpagicConstants._INTERNAL_EVENT_PROCESS_ID);

					if ((exchangeID != null) && (processId != null)) {
						dbManager.relateProcessToServiceInstance(exchangeID, processId);
					}
				}
				return;
			}
			
			String exchangeID = (String) event.getProperty(SpagicConstants.EXCHANGE_ID);
			String correlationID = (String) event.getProperty(SpagicConstants.EXCHANGE_PROPERTY + "." + SpagicConstants.CORRELATION_ID);
			String status = (String) event.getProperty(SpagicConstants.EXCHANGE_STATUS);
			String sender = (String) event.getProperty(SpagicConstants.EXCHANGE_PROPERTY + "." + SpagicConstants.SPAGIC_SENDER);
			String target = (String) event.getProperty(SpagicConstants.EXCHANGE_PROPERTY + "." + SpagicConstants.SPAGIC_TARGET);
			String inBody  = (String) event.getProperty(SpagicConstants.INBODY);
			String outBody  = (String) event.getProperty(SpagicConstants.OUTBODY);
			String faultBody  = (String) event.getProperty(SpagicConstants.FAULTBODY);
			String errorMsg  = (String) event.getProperty(SpagicConstants.EXCHANGE_ERROR);		

			Service senderService = dbManager.getServiceById(sender);
			if (senderService == null) {
				logger.debug("Sender service: " + sender + " not found");
				return;
			}
			
			Service targetService = dbManager.getServiceById(target);
			if (targetService == null) {
				logger.debug("Target service: " + target + " not found");
				return;
			}
			
			if (inBody == null) {
				logger.warn("Input body not found");
				return;
			}
			
			// Search for the service instance with specified exchange id
			ServiceInstance senderServiceInstance = dbManager.getServiceInstanceByCorrelationId(sender, correlationID);
			ServiceInstance targetServiceInstance = dbManager.getServiceInstanceByCorrelationId(target, correlationID);
			
			if (status == SpagicConstants.STATUS_DONE) {

				Date endDate = new Date();
				
				if (senderServiceInstance != null){
					senderServiceInstance.setEnddate(endDate);
					senderServiceInstance.setState(ServiceInstanceStateConstants.SERVICE_DONE);
					dbManager.updateServiceInstance(senderServiceInstance);
				}
				if (targetServiceInstance != null){
					targetServiceInstance.setEnddate(endDate);
					targetServiceInstance.setState(ServiceInstanceStateConstants.SERVICE_DONE);
					dbManager.updateServiceInstance(targetServiceInstance);
				}
				
			} else if (status == SpagicConstants.STATUS_ACTIVE) {
				
				if (outBody != null) {
					// Response message
					if ((senderServiceInstance == null) || (targetServiceInstance == null)) {
						logger.warn("Service instances not found");
						return;
					}
					
					// Save the service response and change the process state
					senderServiceInstance.setResponse(outBody);
					senderServiceInstance.setState(ServiceInstanceStateConstants.SERVICE_DONE);
					dbManager.updateServiceInstance(senderServiceInstance);
					
				} else {
					// Request message
					if (senderServiceInstance == null) {
						
						// Create the service instance only if monitored
						if (senderService.isMonitorEnabled()) {
							// Input message not available, if not provided by the component itself
							senderServiceInstance = dbManager.createServiceInstance(sender, exchangeID + "-0", correlationID, targetServiceInstance, null, null);				
						} else {
							logger.debug("Sender service: " + sender + " not monitored");
						}
						
					}
					
					if (targetServiceInstance == null) {

						// Create the service instance only if monitored
						if (targetService.isMonitorEnabled()) {
							targetServiceInstance = dbManager.createServiceInstance(target, exchangeID, correlationID, null, inBody, null);
						} else {
							logger.debug("Target service: " + target + " not monitored");
						}

						// Update the start service instance with the target service instance
						if (senderServiceInstance != null) {
							senderServiceInstance.setTargetServiceInstance(targetServiceInstance);
							dbManager.updateServiceInstance(senderServiceInstance);
						}
					}
				}			
				
			} else if (status == SpagicConstants.STATUS_ERROR) {
				if ((senderServiceInstance == null) || (targetServiceInstance == null)) {
					logger.error("Service instances not found");
					return;
				}
				
				Date endDate = new Date();
				
				senderServiceInstance.setState(ServiceInstanceStateConstants.SERVICE_FAULTED);
				senderServiceInstance.setEnddate(endDate);
				senderServiceInstance.setResponse(outBody);
				senderServiceInstance.setError((faultBody != null) ? faultBody : errorMsg);
				dbManager.updateServiceInstance(senderServiceInstance);
				targetServiceInstance.setState(ServiceInstanceStateConstants.SERVICE_FAULTED);
				targetServiceInstance.setEnddate(endDate);
				dbManager.updateServiceInstance(targetServiceInstance);				
				
			} else {
				logger.warn("Unknown state: " + status);
			}
		} catch (Throwable th) {
			logger.error("Error monitoring event", th);
		}		
	}

	public IDatabaseManager getDatabaseManager() {
		return this.dbManagerAR.get();
	}
}
