/*******************************************************************************
 * Copyright (c) 2010 Engineering Group.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
/* This file contains code modified from Apache code */
package org.eclipse.ebpm.soap.core;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.ListIterator;

import org.eclipse.ebpm.soap.api.Interceptor;
import org.eclipse.ebpm.soap.api.InterceptorChain;
import org.eclipse.ebpm.soap.api.Message;
import org.slf4j.LoggerFactory;

/**
 * A PhaseInterceptorChain orders Interceptors according 
 * to the before & after properties on an Interceptor.
 */
public class PhaseInterceptorChain implements InterceptorChain {

    private static final org.slf4j.Logger LOG = LoggerFactory.getLogger(PhaseInterceptorChain.class);
    
    private final List<Interceptor> interceptors = new ArrayList<Interceptor>();
    
    public PhaseInterceptorChain() {
    }

    public void add(Iterable<? extends Interceptor> newhandlers) {
        if (newhandlers == null) {
            return;
        }
        for (Interceptor handler : newhandlers) {
            add(handler);
        }
    }
    
    public Iterable<Interceptor> getInterceptors() {
        return interceptors;
    }

    public void add(Interceptor i) {
        if (LOG.isDebugEnabled()) {
            LOG.debug("Adding interceptor " + i.getId());
        }
        insertInterceptor(i);
    }

    /**
     * Invokes each phase's handler in turn.
     * 
     * @param context
     * @throws Exception
     */
    public void doIntercept(Message message) {
        ListIterator<Interceptor> iterator = getState(message);
        
        if (iterator == null) {
            iterator = interceptors.listIterator();
            setState(message, iterator);
        }

        try {
            while (iterator.hasNext()) {
                Interceptor currentInterceptor = iterator.next();
                if (LOG.isDebugEnabled()) {
                    LOG.debug("Invoking handleMessage on interceptor " + currentInterceptor.getId());
                }
                currentInterceptor.handleMessage(message);
            }
        } catch (RuntimeException ex) {
            if (LOG.isInfoEnabled()) {
                LOG.info("Interceptor has thrown exception, unwinding now", ex);
            }
            message.setContent(Exception.class, ex);
            // Unwind
            while (iterator.hasPrevious()) {
                Interceptor currentInterceptor = iterator.previous();
                if (LOG.isDebugEnabled()) {
                    LOG.debug("Invoking handleFault on interceptor " + currentInterceptor.getId());
                }
                currentInterceptor.handleFault(message);
            }
            throw ex;
        }
    }
    
    @SuppressWarnings("unchecked")
    protected ListIterator<Interceptor> getState(Message message) {
        Object state = message.get(this.toString());
        return (ListIterator<Interceptor>) state;
    }
    
    protected void setState(Message message, ListIterator<Interceptor> state) {
        message.put(this.toString(), state);
        message.put(InterceptorChain.class, this);
    }
    
    protected void insertInterceptor(Interceptor interc) {
        if (interceptors.size() == 0) {
            interceptors.add(interc);
            return;
        }
        int begin = -1;
        int end = interceptors.size();
        Collection before = interc.getBefore();
        Collection after = interc.getAfter();
        for (int i = 0; i < interceptors.size(); i++) {
            Interceptor cmp = interceptors.get(i);
            if (cmp.getId() == null) {
                continue;
            }
            if (before.contains(cmp.getId()) && i < end) {
                end = i;
            }
            if (cmp.getBefore().contains(interc.getId()) && i > begin) {
                begin = i;
            }
            if (after.contains(cmp.getId()) && i > begin) {
                begin = i;
            }
            if (cmp.getAfter().contains(interc.getId()) && i < end) {
                end = i;
            }
        }
        if (end < begin + 1) {
            throw new IllegalStateException("Invalid ordering for interceptor " + interc.getId());
        }
        interceptors.add(end, interc);
    }

    
}
