/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.components.bpm.deploy.utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.xml.namespace.QName;

import org.dom4j.Document;
import org.dom4j.XPath;
import org.dom4j.io.SAXReader;
import org.dom4j.xpath.DefaultXPath;
import org.eclipse.ebpm.constants.SpagicConstants;
import org.eclipse.stp.im.BasicProperty;
import org.eclipse.stp.im.Process;
import org.eclipse.stp.im.Property;
import org.eclipse.stp.im.ServiceBinding;
import org.eclipse.stp.im.Step;
import org.eclipse.stp.im.constants.ImConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.spagic3.components.bpm.deploy.DeployPreferenceConstants;

public class DeploymentUtils {
	
	private static final Logger logger = LoggerFactory.getLogger(DeploymentUtils.class);
	
	public static final String PREFIX_NAMESPACE = "urn:eng:spagic:processes:".intern();
	public static final String SPAGIC_PROPERTY_FILE = "spagic.properties".intern();
	public static final String PARAMETER_START = "${".intern();
	public static final String PARAMETER_END = "}".intern();
	public static final String VERSION_LABEL = "_v_".intern();
	
	public static String PROCESSES_FOLDER = "processes";
	public static String IM_SUB_FOLDER = "im";
	public static String SCHEMA_SUB_FOLDER = "mtd";
	public static String GRAPH_SUB_FOLDER = "graph";
	public static String JPDL_SUB_FOLDER = "jpdl";
	public static String DEPLOYMENT_FILE_PREFERENCE = "deploy.properties";
	
	public static final String FORM_SCHEMA_PROPERTY = "FormSchema";
	
	public static final String START_SERVICE_NAME = "StartService".intern();
	public static final String END_SERVICE_NAME = "EndService".intern();
	
	

	public static String getBasePrefixNameSpace(Properties prefs){
		String basePrefixNameSpace = prefs.getProperty(DeployPreferenceConstants.P_SPAGIC_BASE_NAME_SPACE);
		return basePrefixNameSpace != null ? basePrefixNameSpace : PREFIX_NAMESPACE;
	}
	
	public static String getNameSpacePrefixForProcess(Process aProcess){
		return aProcess.getName()+VERSION_LABEL+ getProcessVersion(aProcess);
	}
	public static String getNameSpaceForProcess(Process aProcess, Properties prefs){
		return getBasePrefixNameSpace(prefs)+aProcess.getName()+":v"+getProcessVersion(aProcess);
	}
	public static QName getQNameForProcess(Process aProcess,Properties prefs){
		String localPart = aProcess.getName()+VERSION_LABEL+getProcessVersion(aProcess);
		return new QName(getNameSpaceForProcess(aProcess, prefs), localPart);
	}
	
	public static QName getQNameForStep(Process aProcess, Step aStep, Properties prefs){
		String processNamespace;
		if (aProcess.getProperties().get(ImConstants.PROPERTY_NAMESPACE) != null)
			processNamespace = (String)aProcess.getProperties().get(ImConstants.PROPERTY_NAMESPACE).getValue();
		else
			processNamespace = DeploymentUtils.getNameSpaceForProcess(aProcess, prefs);
		return  new QName(processNamespace, getStepLocalNamePart(aProcess, aStep) );
	}
	
	public static String getStepLocalNamePart(Process aProcess, Step aStep){
		String localPart = aProcess.getName() + "." + aStep.getName()+VERSION_LABEL+getProcessVersion(aProcess);
		return localPart;
	}
	
	public static String getStepNameInProcess(Process aProcess, Step aStep){
		return getNameSpacePrefixForProcess(aProcess) + ":" + getStepLocalNamePart(aProcess, aStep);		
	}
	
	public static String getProcessVersion(Process aProcess){
		Property prop = aProcess.getProperties().get(ImConstants.PROCESS_VERSION);
		return (prop != null ? (String)prop.getValue() : "0");
	}
	
	/**
	 * 
	 * @param s
	 * @return the right service binding for the runtime we're using during the transformation
	 */
	public static ServiceBinding getRightStepBinding(Step s, String runtimeID) {
		
		Property runtimPropertyForServiceBinding = null;
		for (ServiceBinding sb : s.getStepbindings()){
			runtimPropertyForServiceBinding = sb.getProperties().get(ImConstants.IM_STEP_BINDING_RUNTIME_ID);
			if (runtimPropertyForServiceBinding != null){
				if (runtimPropertyForServiceBinding.getValue().equals(runtimeID)){
					return sb;
				}
			}else{
				logger.warn("ServiceBinding["+sb.getName()+"] -> for Step["+s.getName()+"] does not have a property called "+ ImConstants.IM_POOL_RUNTIME_ID);
			}
		}
		logger.warn("Not found for Step["+s.getName()+"] a binding for runtime "+ runtimeID);
		return null;
		
	}
	
	
	
	
	
	/**
	 * Return the property file name containing the dynamic parameters to use for
	 * a specific process.
	 * @param aProcess The IM process to check
	 * @return The property file name
	 */
	public static String getPropertyFileName(Process aProcess) {
		if (isDeployableConfigurable(aProcess)) {
			// The process contains some parameters: use a specific property file
			// for that process
			return getNameSpacePrefixForProcess(aProcess) + ".properties";
		} else {
			// The process doesnt contains parameters: use a fixed empty property file
			return SPAGIC_PROPERTY_FILE;
		}
	}
	
	/**
	 * Checks if a component use a dynamic configuration based on properties files.
	 * @param aProcess The IM process to check
	 * @return true if the process contains at least one dynamic parameter
	 */
	public static boolean isDeployableConfigurable(Process aProcess) {
		
		for (Step aStep : aProcess.getSteps()) {
			for (Map.Entry<String, Property> entry : aStep.getProperties()) {
				Property property = entry.getValue();
				String propertyValue = (String)property.getValue();
				if (propertyValue != null) {
					int startIndex = propertyValue.indexOf(PARAMETER_START);
					if ((startIndex != -1) && (propertyValue.indexOf(PARAMETER_END, startIndex) != -1)) {
						return true;
					}
				}
			}
		}
		
		return false;
	}
	
	public static boolean isStartStep(Step step) {
		
		BasicProperty bp = (BasicProperty)step.getProperties().get(ImConstants.IS_STEP_START_EVENT);
		return ( bp != null && bp.getValue().equalsIgnoreCase("true") );
	}
	
	public static boolean isEndStep(Step step) {
		BasicProperty bp = (BasicProperty)step.getProperties().get(ImConstants.IS_STEP_END_EVENT);
		return ( bp != null && bp.getValue().equalsIgnoreCase("true") );
	}
	
	
	
	
	
	
	public static String getRelativeValue(String value){
		if (value.indexOf("/") > 0)
			return value.substring(value.lastIndexOf("/") + 1);
		else
			return value;
	}
	
	public static String getGraphFileLocation(String processName){
		
		
		String schemaFileLocation = System
				.getProperty(SpagicConstants.SPAGIC_HOME_PROPERTY)
				+ File.separator
				+ PROCESSES_FOLDER
				+ File.separator
				+ GRAPH_SUB_FOLDER
				+ File.separator
				+ processName + "." + GRAPH_SUB_FOLDER;

		return schemaFileLocation;
	}
	
	public static String getSchemaFileLocation(String schemaValue){
		
		String realSchemaValue = getRelativeValue(schemaValue);
		
		String schemaFileLocation = System
				.getProperty(SpagicConstants.SPAGIC_HOME_PROPERTY)
				+ File.separator
				+ PROCESSES_FOLDER
				+ File.separator
				+ SCHEMA_SUB_FOLDER
				+ File.separator
				+ realSchemaValue;

		return schemaFileLocation;
	}
	
	
	public static String getImFileLocation(String processName) {

		String imFileLocation = System
				.getProperty(SpagicConstants.SPAGIC_HOME_PROPERTY)
				+ File.separator
				+ PROCESSES_FOLDER
				+ File.separator
				+ IM_SUB_FOLDER
				+ File.separator
				+ processName
				+ "."
				+ IM_SUB_FOLDER;

		return imFileLocation;
	}
	
	public  static String getJPDLFileLocation(String processName){
		
		String jpdlFileLocation = System
				.getProperty(SpagicConstants.SPAGIC_HOME_PROPERTY)
				+ File.separator
				+ PROCESSES_FOLDER
				+ File.separator
				+ JPDL_SUB_FOLDER
				+ File.separator
				+ processName
				+ "."
				+ JPDL_SUB_FOLDER;
		
		return jpdlFileLocation;
		
	}
	
	public  static String getDeploymentPreferencesFileLocation(){
		
		String jpdlFileLocation = System
				.getProperty(SpagicConstants.SPAGIC_HOME_PROPERTY)
				+ File.separator
				+ DEPLOYMENT_FILE_PREFERENCE;
		
		return jpdlFileLocation;
		
	}
	
	public static String evalXPathAsString(Document document, String xpath) {
		try {
			XPath xPath = new DefaultXPath(xpath);
			//xPath.setNamespaceURIs(namespaceMap);
			return xPath.valueOf(document);
		} catch (Exception e) {
			return null;
		}
	}

	public static Document fetchDocument(File schemaFile) {
		final SAXReader xmlReader = new SAXReader();
		Document document = null;
		InputStream is = null;
		try {
			is = new FileInputStream(schemaFile);
			document = xmlReader.read(is);
		} catch (Exception ex) {
			logger.error(ex.getMessage(), ex);
		} finally {
			if (is != null) {
				try {
					is.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
		return document;
	}
}
