/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.components.bpm.monitoring;

import java.util.Date;
import java.util.List;

import org.hibernate.Session;
import org.hibernate.Transaction;
import org.jbpm.context.exe.ContextInstance;
import org.jbpm.graph.def.Node;
import org.jbpm.graph.exe.Token;
import org.jbpm.graph.log.ProcessInstanceEndLog;
import org.spagic.metadb.base.ProcessInstanceStateConstants;
import org.spagic.metadb.base.TransitionStateConstants;
import org.spagic.metadb.dbutils.AuditDBUtils;
import org.spagic.metadb.model.LogJournal;
import org.spagic.metadb.model.Process;
import org.spagic.metadb.model.ProcessInstance;
import org.spagic.metadb.model.Step;
import org.spagic.metadb.model.TransitionInstance;
import org.spagic.monitoring.core.TransitionStateCache;
import org.spagic.monitoring.jbpm.MonitorServiceJBPM;
import org.spagic.workflow.api.jbpm.ProcessEngine;
import org.spagic.workflow.logging.log.FaultLog;
import org.spagic3.components.bpm.BPMContextSingleton;

public class MonitorJBPMSpagic3 extends MonitorServiceJBPM {
	
	@Override
	protected void processFaultLog(FaultLog faultLog) {
		
		Token token = faultLog.getToken();
		org.jbpm.graph.exe.ProcessInstance jBPMProcessInstance = token.getProcessInstance();
		String nodeName = faultLog.getNodeName();
		
		Session aSession = null;
		Transaction tx = null;
		String xmlMessage = null;
		String orchestrationServiceId = null;
		boolean avoidConnectorOutput = false;
		String encodedAttachments = null;
		String messageHeadersAsXML = null;
		ProcessInstance spagicProcessInstance = null;
		int originalProcessState = -1;
		try {
			// Retrieve current session
			aSession = getSession();
			tx = aSession.beginTransaction();
			
			
			// Retrieve Sign. Variables
			ContextInstance contextInstance = jBPMProcessInstance.getContextInstance();
			xmlMessage = (String)contextInstance.getVariable(BPMContextSingleton.XML_MESSAGE);
			encodedAttachments = (String)contextInstance.getVariable(BPMContextSingleton.ATTACHMENTS);
			messageHeadersAsXML = (String)contextInstance.getVariable(BPMContextSingleton.MESSAGE_HEADERS);
			orchestrationServiceId = (String)contextInstance.getVariable(BPMContextSingleton.ORCHESTRATION_SERVICE_ID);
			String avoidConnectorOutputString = (String)contextInstance.getVariable(BPMContextSingleton.AVOID_CONNECTOR_OUTPUT);
			
			if (avoidConnectorOutputString != null){
				avoidConnectorOutput = Boolean.valueOf(avoidConnectorOutputString);
			}
			
			
			// Retrieve Spagic process instance
			String correlationID = ProcessEngine.createSpagicCorrelationId(jBPMProcessInstance);
			spagicProcessInstance = AuditDBUtils.getProcessInstanceByIdCorrelation(aSession, correlationID);
			Process spagicProcess = spagicProcessInstance.getProcess();
			
			// Retrieve Spagic step
			Step step = getSpagicStepFromJBPMNode(spagicProcess, nodeName, aSession);
			
			// Close with fault all active transitions (there should be one)
			List<TransitionInstance> transitionInstances = AuditDBUtils
					.getTransitionInstancesByProcessInstanceIdAndTargetStep(aSession, spagicProcessInstance
							.getIdProcessInstance(), step.getIdStep());
			for (TransitionInstance transitionInstance : transitionInstances) {
				
				if (TransitionStateConstants.JBPM_AVAILABLE_ACTIVITY == transitionInstance.getTransitionState().getIdState()) {
						
					transitionInstance.setTransitionEnd(faultLog.getDate());
					long endTime = transitionInstance.getTransitionEnd().getTime();
					long startTime = transitionInstance.getTransitionStart().getTime();
					transitionInstance.setTransitionExecutionTime(endTime - startTime);
					transitionInstance.setTransitionState(TransitionStateCache
							.getTransitionState(TransitionStateConstants.JBPM_FAULTED_ACTIVITY));
					
					aSession.save(transitionInstance);
					
					// Trace all variables
					traceVariables(jBPMProcessInstance.getContextInstance(), aSession, transitionInstance);

					// Log the fault, so that it shows up in the Relevant Data tab, side
					// by side with the XML_MESSAGE. The String rappresentation of the faultLog
					// seems the more appropriate information to show.
					logFault(aSession, transitionInstance, faultLog.toString());
				}
			}
			
			
			// Ensure that the process instance state is updated only once: only the first time the process
			// instance fails, we have the data necessary to restart it. The subsequent times, these data are lost
			// because the process instance has already ended by JBPM
			originalProcessState = spagicProcessInstance.getState();
			if (originalProcessState != ProcessInstanceStateConstants.PROCESS_FAULTED) {
				spagicProcessInstance.setState(ProcessInstanceStateConstants.PROCESS_FAULTED);
			}
			
			aSession.flush();
			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			e.printStackTrace();
        	log.error("traceExchange ", e);
        	
		} finally {
			try {
				closeSession(aSession);
			} catch (Throwable th) {
	        	log.error("Error closing session", th);
			}
			
			// This method can be called multiple times for the same process instance:
			// we must ensure that the callback is called only once.
			if ((originalProcessState != -1) && (originalProcessState != ProcessInstanceStateConstants.PROCESS_FAULTED)) {
				BPMContextSingleton.callBack(orchestrationServiceId, 
						spagicProcessInstance.getIdProcessInstance(), 
						xmlMessage,
						encodedAttachments,
						messageHeadersAsXML,
						/*fault*/ true, 
						avoidConnectorOutput);			
			}
		}
		
	}
	
	private void logFault(Session aSession, TransitionInstance transitionInstance, String value ) {
		LogJournal messageInfo = new LogJournal();
		messageInfo.setName("ERROR");
		messageInfo.setTransitionInstance(transitionInstance);
		messageInfo.setTransitionState(transitionInstance.getTransitionState());
		messageInfo.setDatelog((transitionInstance.getTransitionEnd() != null) ? transitionInstance.getTransitionEnd() : transitionInstance.getTransitionStart());
		messageInfo.setLog(value);
		messageInfo.setIdLogJournalType(1);

		aSession.save(messageInfo);

	}
	@Override
	protected void processProcessInstanceEndLog(ProcessInstanceEndLog processLog) {
		
		log.info("************************************************************************");
		log.info("********************** PROCESS INSTANCE END LOG ************************");
		log.info("************************************************************************");
		
		// A quale istanza di processo di riferisce questo log ?
		Token token = processLog.getToken();
		org.jbpm.graph.exe.ProcessInstance jBPMProcessInstance = token.getProcessInstance();
		String xmlMessage = null;
		String orchestrationServiceId = null;
	
		ProcessInstance spagicProcessInstance = null;
		Session aSession = null;
		Transaction tx = null;
		boolean fault = false;
		boolean avoidConnectorOutput = false;
		String encodedAttachments = null;
		String messageHeadersAsXML = null;
		try {
			// Retrieve current session
			aSession = getSession();
			tx = aSession.beginTransaction();

			// Retrieve Spagic process instance
			String correlationID = ProcessEngine.createSpagicCorrelationId(jBPMProcessInstance);
			spagicProcessInstance = AuditDBUtils.getProcessInstanceByIdCorrelation(aSession, correlationID);
			Process spagicProcess = spagicProcessInstance.getProcess();
			
		
			ContextInstance contextInstance = jBPMProcessInstance.getContextInstance();
			
			// Retrieve Sign. Variables
			xmlMessage = (String)contextInstance.getVariable(BPMContextSingleton.XML_MESSAGE);
			encodedAttachments = (String)contextInstance.getVariable(BPMContextSingleton.ATTACHMENTS);
			messageHeadersAsXML = (String)contextInstance.getVariable(BPMContextSingleton.MESSAGE_HEADERS);
			orchestrationServiceId = (String)contextInstance.getVariable(BPMContextSingleton.ORCHESTRATION_SERVICE_ID);
			String avoidConnectorOutputString = (String)contextInstance.getVariable(BPMContextSingleton.AVOID_CONNECTOR_OUTPUT);
			
			if (avoidConnectorOutputString != null){
				avoidConnectorOutput = Boolean.valueOf(avoidConnectorOutputString);
			}
			
			
			
			// Retrieve the end node
			Node endNode = jBPMProcessInstance.getRootToken().getNode();
			String endNodeName = endNode.getName();
			Date processEndDate = jBPMProcessInstance.getRootToken().getEnd();
			
			// Retrieve Spagic target steps
			Step targetStep = getSpagicStepFromJBPMNode(spagicProcess, endNodeName, aSession);

			fault = ( spagicProcessInstance.getState() == ProcessInstanceStateConstants.PROCESS_FAULTED ) ;
			List<TransitionInstance> transitionInstances = AuditDBUtils.getTransitionInstancesByProcessInstanceIdAndTargetStep(aSession, spagicProcessInstance.getIdProcessInstance(), targetStep.getIdStep());
			if (!fault){
				for (TransitionInstance previousTransitionInstance : transitionInstances) {
					if (TransitionStateConstants.JBPM_FAULTED_ACTIVITY
							== previousTransitionInstance.getTransitionState().getIdState()) {
						fault = true;
						break;
					}
				}
				
				if (fault) {
					spagicProcessInstance.setState(ProcessInstanceStateConstants.PROCESS_FAULTED);
				} else {
					spagicProcessInstance.setState(ProcessInstanceStateConstants.PROCESS_DONE);
				}
			}
			
			
			Date endDate = jBPMProcessInstance.getEnd();
			Date startDate = spagicProcessInstance.getStartdate();
			spagicProcessInstance.setEnddate(endDate);
			spagicProcessInstance.setExecutionTime(endDate.getTime() - startDate.getTime());
			
			aSession.save(spagicProcessInstance);
			
			if(!fault) {
				for (TransitionInstance previousTransitionInstance : transitionInstances) {
					
					previousTransitionInstance.setTransitionEnd(processEndDate);
					previousTransitionInstance.setTransitionExecutionTime(previousTransitionInstance.getTransitionEnd().getTime() - previousTransitionInstance.getTransitionStart().getTime());
					previousTransitionInstance.setTransitionState(TransitionStateCache.getTransitionState(TransitionStateConstants.JBPM_DONE_ACTIVITY));
					
					aSession.save(previousTransitionInstance);
					
					// Trace all variables
					traceVariables(jBPMProcessInstance.getContextInstance(), aSession, previousTransitionInstance);
				}
			}
			
			aSession.flush();
			tx.commit();
			
		} catch (Exception e) {
			tx.rollback();
        	log.error("traceExchange ", e);
		} finally {
			try {
				closeSession(aSession);
			} catch (Throwable th) {
	        	log.error("Error closing session", th);
			}
			BPMContextSingleton.callBack(orchestrationServiceId, 
										spagicProcessInstance.getIdProcessInstance(), 
										xmlMessage,
										encodedAttachments,
										messageHeadersAsXML,
										fault, 
										avoidConnectorOutput);
		}
	}


	/*
	xmlMessage = (String)jBPMProcessInstance.getContextInstance().getVariable(BPMContextSingleton.XML_MESSAGE);
	orchestrationServiceId = (String)jBPMProcessInstance.getContextInstance().getVariable(BPMContextSingleton.ORCHESTRATION_SERVICE_ID);
	
	String processTerminated = (String)jBPMProcessInstance.getContextInstance().getVariable(SpagicConstants.WF_IS_PROCESS_TERMINATED);
	
	if (processTerminated != null)
		isProcessTerminated = Boolean.valueOf(processTerminated);
	*/
}
