/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.connectors.tcp;

import java.net.InetSocketAddress;

import org.apache.mina.common.ConnectFuture;
import org.apache.mina.common.IoConnector;
import org.apache.mina.common.IoFutureListener;
import org.apache.mina.common.IoSession;
import org.apache.mina.common.RuntimeIOException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Class <code>SpagicConnectFuture</code> An enhanced ConnectFuture
 * with accessible handler to get a cleaner code for waiting for a
 * reply to data sent.
 *
 * @author <a href="mailto:saint@eng.it">Gian Uberto Lauri</a>
 * @version $Revision$
 */
public class SpagicConnectFuture implements ConnectFuture, Notifiable {

	private static final Logger log = LoggerFactory.getLogger(SpagicConnectFuture.class);
	
	ConnectFuture decorated;
	TCPClientProtocolHandler handler;
	Object waitingPlace;
	Long timeout;
	Object reply;
	
	/**
	 * Creates a new instance of <code>SpagicConnectFuture</code> .
	 *
	 * @param connector an <code>IoConnector</code> used to create the
	 * connection
	 * @param config a <code>TCPBCConfig</code> used to create the handler
	 * @param remoteHost a <code>String</code> the remote host to connect
	 * to
	 * @param remotePort an <code>Integer</code> the port in the remote
	 * host to connect to
	 */
	public SpagicConnectFuture(IoConnector connector, 
							   TCPBCConfig config, 
							   String remoteHost, 
							   Integer remotePort) {
		handler = new TCPClientProtocolHandler(config);
		decorated = connector.connect(
				new InetSocketAddress(remoteHost, remotePort),
				handler);
		waitingPlace = new Object();
		handler.setNotifyMePlease(this);
		timeout = config.getResponseTimeout();

	}

	@Override
	/**
	 * <code>getSession</code> see ConnectFuture#getSession()
	 *
	 * @return an <code>IoSession</code> see ConnectFuture#getSession()
	 * @exception RuntimeIOException see ConnectFuture#getSession()
	 * @see ConnectFuture#getSession()
	 */
	public IoSession getSession() throws RuntimeIOException {
		return decorated.getSession();
	}

	@Override
	/**
	 * <code>isConnected</code> see ConnectFuture#isConnected()
	 *
	 * @return a <code>boolean</code> see ConnectFuture#isConnected()
	 * @see ConnectFuture#isConnected()
	 */
	public boolean isConnected() {
		return decorated.isConnected();
	}

	@Override
	/**
	 * <code>setException</code> see ConnectFuture#setException()
	 *
	 * @param arg0 a <code>Throwable</code> see
	 * ConnectFuture#setException(). In my humble opinion the choice
	 * of Throwable stinks, if you agree blame Apache Software
	 * Founddation.
	 * @see ConnectFuture#setException()
	 */
	public void setException(Throwable arg0) {
		decorated.setException(arg0);
	}

	@Override
	/**
	 * <code>setSession</code> see ConnectFuture#setSession(IoSession)
	 *
	 * @param arg0 an <code>IoSession</code> see
	 * ConnectFuture#setSession(IoSession) 
	 * @see ConnectFuture#setSession(IoSession)
	 */
	public void setSession(IoSession arg0) {
		decorated.setSession(arg0);
	}

	@Override
	/**
	 * <code>addListener</code> see
	 * IoFuture#addListener(IoFutureListener) 
	 *
	 * @param arg0 an <code>IoFutureListener</code> see
	 * IoFuture#addListener(IoFutureListener)
	 * @see IoFuture#addListener(IoFutureListener)
	 */
	public void addListener(IoFutureListener arg0) {
		decorated.addListener(arg0);
	}

	@Override
	/**
	 * <code>getLock</code> see IoFuture#getLock()
	 *
	 * @return an <code>Object</code> see IoFuture#getLock()
	 * @see IoFuture#getLock()
	 */
	public Object getLock() {
		return decorated.getLock();
	}

	@Override
	/**
	 * <code>isReady</code> see IoFuture#isReady()
	 *
	 * @return a <code>boolean</code> see IoFuture#isReady()
	 * @see IoFuture#isReady()
	 */
	public boolean isReady() {
		return decorated.isReady();
	}

	@Override
	/**
	 * <code>join</code> see IoFuture#join()
	 * @see IoFuture#join()
	 *
	 */
	public void join() {
		decorated.join();
	}

	@Override
	/**
	 * <code>join</code> see IoFuture#join(long)
	 *
	 * @param arg0 a <code>long</code> see IoFuture#join(long)
	 * @return a <code>boolean</code> see IoFuture#join(long)
	 * @see IoFuture#join(long)
	 */
	public boolean join(long arg0) {
		return decorated.join(arg0);
	}

	@Override
	/**
	 * <code>removeListener</code> see
	 * IoFuture#removeListener(IoFutureListener)
	 *
	 * @param arg0 an <code>IoFutureListener</code> see
	 * IoFuture#removeListener(IoFutureListener)
	 * @see
	 * IoFuture#removeListener(IoFutureListener)
	 */
	public void removeListener(IoFutureListener arg0) {
		decorated.removeListener(arg0);
	}


	public Object waitForResponse(IoSession session) {

	
		synchronized(waitingPlace) {
			if (reply == null) {
				try {
					reply = null;
					waitingPlace.wait(timeout);
				}
				catch(InterruptedException inex) {
				}
			}
		}
		log.debug((reply == null) ? "No reply" : reply.toString());
		return reply;
	}

	@Override
	public void notifyMe(Object message) {
		synchronized(waitingPlace) {
			reply = message;
			waitingPlace.notify();
			log.debug("Notified" + message);
		}
	}

	public void resetReply() {
		synchronized(waitingPlace) {
			reply = null;
		}
	}
}
